!***************************************************************
!*
!*		Module for input/output
!*
!***************************************************************
 MODULE io_parameters
   IMPLICIT NONE
   SAVE
!
!***  File logical units
!
	  integer(kind=4), parameter  :: nlog = 1      ! log          file
	  integer(kind=4), parameter  :: ninp = 10     ! input        file
	  integer(kind=4), parameter  :: nmet = 11     ! input        file
	  integer(kind=4), parameter  :: ndia = 12     ! input        file
!
!***  File names
!
      character(len=250) :: flog,finp,ftop,frou,fmet,fdia,dump ! input file names
!
    CONTAINS
!
!
!
      subroutine get_input_int4(fname,block,line,value,nval,istat,message)
!**************************************************************************
!*
!*    Gets nval integer*4 inputs from the file fname
!*
!*    INPUT:
!*    character*(*)   fname    Name of the file
!*    character*(*)   block    Block to search
!*    character*(*)   line     Line block to search
!*    integer*4       nval     Number of integers to read                     
!*
!*    OUTPUT:
!*    integer*4       istat    0 if OK
!*    integer*4       value    Values of the nval integers read
!*    character*100   message  Output message with the error description       
!*
!**************************************************************************
      implicit none
!
      character(len=100) :: message
      character*(*)      :: fname,block,line
	  integer(kind=4)    :: nval,istat
      integer(kind=4)    :: value
!
      logical            :: linefound,blockfound
	  integer(kind=4)    :: nword,npar,ival
	  character(len=256) :: card
	  character(len=256) :: words(128)
      real     (kind=8 ) :: param(128)
!
!***  Initializations
!
      istat = -1
!
!***  Opens the file
!
      open(90,FILE=TRIM(fname),STATUS='old',ERR=101)
!
!***  Search the line
!
      blockfound = .false.
      linefound  = .false.
      do while(.not.linefound)
         do while(.not.blockfound) 
             read(90,'(a256)',END=102) card
             call decode(card,words,param,nword,npar)
             if(words(1)(1:LEN_TRIM(block)).eq.block(1:LEN_TRIM(block))) blockfound=.true.
         end do
         read(90,'(a256)',END=103) card
         call decode(card,words,param,nword,npar)
         if(words(1)(1:LEN_TRIM(line)).eq.line(1:LEN_TRIM(line))) linefound = .true.
      end do
!                 
      if(npar.lt.nval) goto 104
!
!      do ival = 1,nval
!	     value(ival) = INT(param(ival))
!      end do
       value  = INT(param(1))
!
!***  Successful end
!	 
      close(90)
      istat = 0
      message = ' '
	  return 
!
!***  List of errors
!
  101 message = 'get_input_int4: error opening the input file '//TRIM(fname)
      return
  102 message = 'get_input_int4: block '//TRIM(block)//' not found in the input file' 
      return
  103 message = 'get_input_int4: line '//TRIM(line)//' not found in the input file' 
      return
  104 message = 'get_input_int4: too few parameters in line '//TRIM(line) 
      return
      end subroutine get_input_int4
!
!
!
      subroutine get_input_rea4(fname,block,line,value,nval,istat,message)
!**************************************************************************
!*
!*    Gets nval real*4 inputs from the file fname
!*
!*    INPUT:
!*    character*(*)   fname    Name of the file
!*    character*(*)   block    Block to search
!*    character*(*)   line     Line block to search
!*    integer*4       nval     Number of integers to read                     
!*
!*    OUTPUT:
!*    integer*4       istat    0 if OK
!*    real*4          value    Values of the nval integers read
!*    character*100   message  Output message with the error description       
!*
!**************************************************************************
      implicit none
!
      character(len=100) :: message
      character*(*)      :: fname,block,line
	  integer(kind=4)    :: nval,istat
      real   (kind=4)    :: value
!
      logical            :: linefound,blockfound
      integer  (kind=4)  :: nword,npar,ival
	  character(len=256) :: card
	  character(len=256) :: words(128)
      real     (kind=8)  :: param(128)
!
!***  Initializations
!
      istat = -1
!
!***  Opens the file
!
      open(90,FILE=TRIM(fname),STATUS='old',ERR=101)
!
!***  Search the line
!
      blockfound = .false.
      linefound  = .false.
      do while(.not.linefound)
         do while(.not.blockfound) 
             read(90,'(a256)',END=102) card
             call decode(card,words,param,nword,npar)
             if(words(1)(1:LEN_TRIM(block)).eq.block(1:LEN_TRIM(block))) blockfound=.true.
         end do
         read(90,'(a256)',END=103) card
         call decode(card,words,param,nword,npar)
         if(words(1)(1:LEN_TRIM(line)).eq.line(1:LEN_TRIM(line))) linefound = .true.
      end do
!                 
      if(npar.lt.nval) goto 104
!
!      do ival = 1,nval
!	     value(ival) = SNGL(param(ival))
!      end do
	   value = SNGL(param(1))
!
!***  Successful end
!	 
      close(90)
      istat = 0
      message = ' '
	  return
!
!***  List of errors
!
  101 message = 'Error opening the input file '//TRIM(fname)
      return
  102 message = 'Block '//TRIM(block)//' not found in the input file' 
      return
  103 message = 'Line '//TRIM(line)//' not found in the input file' 
      return
  104 message = 'Number of parameters in line '//TRIM(line)//' is less than required' 
      return
      end subroutine get_input_rea4
!
!
!
      subroutine get_input_cha(fname,block,line,value,nval,istat,message)
!**************************************************************************
!*
!*    Gets nval character inputs from the file fname
!*
!*    NOTE: words are converted to UPPER case
!*
!*    INPUT:
!*    character*(*)   fname    Name of the file
!*    character*(*)   block    Block to search
!*    character*(*)   line     Line block to search
!*    integer*4       nval     Number of integers to read                     
!*
!*    OUTPUT:
!*    integer*4       istat    0 if OK
!*    character*(*)   value    Values of the nval integers read
!*    character*100   message  Output message with the error description       
!*
!**************************************************************************
      implicit none
!
      integer(kind=4)    :: nval,istat
      character(len=100) :: message
      character*(*)      :: fname,block,line
      character*(*)      :: value
!
      logical            :: linefound,blockfound
	  integer(kind=4)    :: nword,npar,ival,j
	  character(len=256) :: card
	  character(len=256) :: words(128)
      real(kind=8)       :: param(128)
!
!***  Initializations
!
      istat = -1
!
!***  Opens the file
!
      open(90,FILE=TRIM(fname),STATUS='old',ERR=101)
!
!***  Search the line
!
      blockfound = .false.
      linefound  = .false.
      do while(.not.linefound)
         do while(.not.blockfound) 
             read(90,'(a256)',END=102) card
             call decode(card,words,param,nword,npar)
             if(words(1)(1:LEN_TRIM(block)).eq.block(1:LEN_TRIM(block))) blockfound=.true.
         end do
         read(90,'(a256)',END=103) card
         call decode(card,words,param,nword,npar)
         if(words(1)(1:LEN_TRIM(line)).eq.line(1:LEN_TRIM(line))) linefound = .true.
      end do
!  
      if((nword-1).lt.nval) goto 104
!
!      do ival = 1,nval
!	     value(ival)(1:LEN_TRIM(words(ival+1))) = words(ival+1)(1:LEN_TRIM(words(ival+1)))
!
!***     Fill the rest with ' '
!
!	     do j=LEN_TRIM(words(ival+1))+1,LEN(value(ival))
!	        value(ival)(j:j)=' '
!	     end do
!	  end do

      value(1:LEN_TRIM(words(2))) = words(2)(1:LEN_TRIM(words(2)))
!
!***     Fill the rest with ' '
!
	     do j=LEN_TRIM(words(2))+1,LEN(value)
	        value(j:j)=' '
	     end do

!
!***  Successful end
!	 
      close(90)
      istat = 0
      message = ' '
	  return
!
!***  List of errors
!
  101 message = 'get_input_cha: error opening the input file '//TRIM(fname)
      return
  102 message = 'get_input_cha: block '//TRIM(block)//' not found in the input file' 
      return
  103 message = 'get_input_cha: line '//TRIM(line)//' not found in the input file' 
      return
  104 message = 'get_input_cha: too few parameters in line '//TRIM(line) 
	  return
      end subroutine get_input_cha
!
!
 END MODULE io_parameters
