      SUBROUTINE ADJUST(U,V,PHI,HTOPO,HBAR,UB,VB)
C
C     DIAGNO   VERSION 1.1  LEVEL 900221  ADJUST
C
      include 'diagno.inc'
C
      COMMON /D5/ NX,NY,NZ,DX,DY,DZ(NZMAX)
      COMMON /D6/ IRD,IWR,IFILE
      DIMENSION U(NXMAX,NYMAX,*),V(NXMAX,NYMAX,*)
      DIMENSION PHI(NXMAX,NYMAX,*),HTOPO(NXMAX,*)
      DIMENSION UB(NYMAX,2,*),VB(NXMAX,2,*)
C
C      THIS ROUTINE ADJUSTS SURFACE WINDS FOR TERRAIN EFFECTS
C
C     INPUTS:  U (R ARRAY)     - GRIDDED X-DIRECTION WIND COMPONENTS
C              V (R ARRAY)     - GRIDDED Y-DIRECTION WIND COMPONENTS
C              HTOPO (R ARRAY) - GRIDDED TERRAIN HEIGHTS
C              HBAR (R ARRAY)  - MODEL LEVELS
C              UB (R ARRAY)    - U-COMPONENT BOUNDARY VALUES
C              VB (R ARRAY)    - V-COMPONENT BOUNDARY VALUES
C
C     OUTPUTS:  U (R ARRAY) - X-DIRECTION WIND COMPONENTS WITH
C                             ADJUSTED SURFACE LAYER WINDS
C               V (R ARRAY) - Y-DIRECTION WIND COMPONENTS WIHT
C                             ADJUSTED SURFACE LAYER WINDS
C
C     ITERATION CRITERIA
C
      DATA ITMAX,EPSI,OVREL/75,0.02,1.5/
      KK = 1
C
C     COMPUTE TERRAIN GRADIENTS AND INITIAL POTENTIAL
C
      DXI=0.5/DX
      DYI=0.5/DY
      FX=DXI/(HBAR)
      FY=DYI/(HBAR)
      DO 20 J=1,NY
      DO 20 I=1,NX
      HTOIM1=HTOPO(I,J)
      HTOJM1=HTOPO(I,J)
      IF(I.GT.1) HTOIM1=HTOPO(I-1,J)
      IF(J.GT.1) HTOJM1=HTOPO(I,J-1)
      HTOIP1=HTOPO(I,J)
      HTOJP1=HTOPO(I,J)
      IF(I.LT.NX) HTOIP1=HTOPO(I+1,J)
      IF(J.LT.NY) HTOJP1=HTOPO(I,J+1)
      DHDX=(HTOIP1-HTOIM1)*FX
      DHDY=(HTOJP1-HTOJM1)*FY
      PHI(I,J,2)=(U(I,J,KK)*DHDX+V(I,J,KK)*DHDY)
   20 CONTINUE
C
C     SET BOUNDARY VALUES FOR PHI
C
      DO 30 J=1,NY
      UB(J,1,1)=PHI(1,J,KK)
      UB(J,2,1)=PHI(NX,J,KK)
   30 CONTINUE
      DO 35 I=1,NX
      VB(I,1,1)=PHI(I,1,KK)
      VB(I,2,1)=PHI(I,NY,KK)
   35 CONTINUE
C
C     SOLVE POISSON EQUATION BY GAUSS-SEIDEL METHOD FOR
C     VELOCITY POTENTIAL
C
      DXSQ=DX*DX
      DYSQ=DY*DY
      DSQ=DXSQ*DYSQ
      FACT=1.0/(2.0*(DXSQ+DYSQ))
      DO 100 IT=1,ITMAX
      DO 90 IDIR=1,4
      ERROR=-1.0E+09
      DO 50 JJ=1,NY
      DO 50 II=1,NX
      GO TO (71,72,73,74),IDIR
   71 I=II
      J=JJ
      GO TO 75
   72 I=NX-II+1
      J=JJ
      GO TO 75
   73 I=II
      J=NY-JJ+1
      GO TO 75
   74 I=NX-II+1
      J=NY-JJ+1
   75 CONTINUE
      XOLD=PHI(I,J,KK)
      PHIIM1=UB(J,1,1)
      IF(I.GT.1)PHIIM1=PHI(I-1,J,KK)
      PHIJM1=VB(I,1,1)
      IF(J.GT.1)PHIJM1=PHI(I,J-1,KK)
      PHIIP1=UB(J,2,1)
      IF(I.LT.NX)PHIIP1=PHI(I+1,J,KK)
      PHIJP1=VB(I,2,1)
      IF(J.LT.NY)PHIJP1=PHI(I,J+1,KK)
      XX=DYSQ*(PHIIP1+PHIIM1)
      YY=DXSQ*(PHIJP1+PHIJM1)
      PHI(I,J,KK) = (1.-OVREL)*PHI(I,J,KK)
     1            + OVREL*FACT*(XX+YY-DSQ*PHI(I,J,2))
      IF(I.EQ.1) UB(J,1,1)=PHI(1,J,KK)
      IF(I.EQ.NX) UB(J,2,1)=PHI(NX,J,KK)
      IF(J.EQ.1) VB(I,1,1)=PHI(I,1,KK)
      IF(J.EQ.NY) VB(I,2,1)=PHI(I,NY,KK)
      IF(XOLD.EQ.0.) GO TO 50
      IF(ABS(XOLD).LT.1.0E-10) GO TO 50
      ERR=ABS((PHI(I,J,KK)-XOLD)/XOLD)
      ERROR=AMAX1(ERR,ERROR)
   50 CONTINUE
   90 CONTINUE
      IF (ERROR.LE.EPSI) GO TO 150
  100 CONTINUE
  150 CONTINUE
C
C     COMPUTE WIND COMPONENTS FROM VELOCITY POTENTIAL
C
      DO 170 J=1,NY
      DO 170 I=1,NX
      PHIIM1=UB(J,1,1)
      PHIJM1=VB(I,1,1)
      PHIIP1=UB(J,2,1)
      PHIJP1=VB(I,2,1)
      IF(I.GT.1) PHIIM1=PHI(I-1,J,KK)
      IF(J.GT.1) PHIJM1=PHI(I,J-1,KK)
      IF(I.LT.NX) PHIIP1=PHI(I+1,J,KK)
      IF(J.LT.NY) PHIJP1=PHI(I,J+1,KK)
      U(I,J,KK)=(PHIIP1-PHIIM1)*DXI+U(I,J,KK)
      V(I,J,KK)=(PHIJP1-PHIJM1)*DYI+V(I,J,KK)
  170 CONTINUE
  200 CONTINUE
C
C     RESET BOUNDARY ARRAY
C
      DO 300 K=1,2
      DO 250 J=1,NY
  250 UB(J,K,1)=0.
      DO 260 I=1,NX
  260 VB(I,K,1)=0.
  300 CONTINUE
      RETURN
      END