      SUBROUTINE FRADJ(U,V,HTOPO,GAMMA,TINF,CRITFN,CELLZB,
     1                  HMAX,TIME)
C
C     DIAGNO  VERSION 1.1  LEVEL 900221  FRADJ  S.DOUGLAS, SAI
C
      include 'diagno.inc'
C
      COMMON /D5/ NX,NY,NZ,DX,DY,DZ(NZMAX)
      COMMON /D6/ IRD,IWR,IFILE
      DIMENSION U(NXMAX,NYMAX,1), V(NXMAX,NYMAX,1), GAMMA(1),
     1          HTOPO(NXMAX,1), HMAX(NXMAX,1), CELLZB(NZPMAX)
C
C     THIS SUBROUTINE IS ADAPTED FROM THE MELSAR MODEL.
C     THE LOCAL FROUDE NUMBER FOR EACH GRID POINT IS CALCULATED
C     IF THIS EXCEEDS A CRITICAL VALUE AND IF THE WIND IS BLOWING
C     TOWRD THE OBSTACLE, THE U AND V WIND COMPONENTS ARE
C     ADJUSTED.
C
C     INPUT:  U (R ARRAY)     - GRIDDED X-DIRECTION WIND COMPONENTS
C             V (R ARRAY)     - GRIDDED Y-DIRECTION WIND COMPONENTS
C             HTOPO (R ARRAY) - GRIDDED TERRAIN HEIGHTS
C             GAMMA (R ARRAY) - TEMPERATURE LAPSE RATES (HOURLY)
C             TINF (R)        - DOMAIN REPRESENTATIVE TEMPERATURE
C             CRITFN (R)      - CRITICAL FROUDE NUMBER
C             HMAX (R ARRAY)  - MAXIMUM TERRAIN HEIGHTS WITHIN A
C                               GIVEN RADIUS
C             TIME (R)        - SIMULATION TIME
C
C     OUTPUT:  U (R ARRAY) - ADJUSTED X-DIRECTION WIND COMPONENTS
C              V (R ARRAY) - ADJUSTED Y-DIRECTION WIND COMPONENTS
C
      DATA ZERO /0./
C
C     COMPUTE THE STABILITY PARAMETER
C
      TAU=-0.01
c      IT = IFIX(TIME/100.) + 1
      IT =1
      GAMMA2=GAMMA(IT) - TAU
C
C     MELSAR FROUDE NUMBER ADJUSTMENT
C
      RPD=0.017453
      DPR=57.296
      DO 100 J=1,NY
      DO 100 I=1,NX
C
C     CALCULATE THE TOPOGRAPHIC GRADIENTS
C
      DXI=0.5/DX
      DYI=0.5/DY
      HTOIM1=HTOPO(I,J)
      HTOJM1=HTOPO(I,J)
      IF (I .GT. 1) HTOIM1=HTOPO(I-1,J)
      IF (J .GT. 1) HTOJM1=HTOPO(I,J-1)
      HTOIP1=HTOPO(I,J)
      HTOJP1=HTOPO(I,J)
      IF (I .LT. NX) HTOIP1=HTOPO(I+1,J)
      IF (J .LT. NY) HTOJP1=HTOPO(I,J+1)
      DELHI=(HTOIP1-HTOIM1)*DXI
      DELHJ=(HTOJP1-HTOJM1)*DYI
      IF (ABS(DELHI) .LT. 0.00015 .AND. ABS(DELHJ) .LT. 0.00015) THEN
        DELHI = 0.
        DELHJ = 0.
      ENDIF
C
      DO 100 K=1,NZ
C
C     CALCULATE THE LOCAL FROUDE NUMBER
C
      SPEED=(U(I,J,K)**2+V(I,J,K)**2)**0.5
      OBSHGT=HMAX(I,J)-(HTOPO(I,J)+CELLZB(K)+DZ(K)/2.)
      IF (OBSHGT .LE. 0.) GO TO 100
      IF (GAMMA2 .LE. 0.) GO TO 100
      FROUDE=SPEED/(((9.8*GAMMA2/TINF)**0.5)*OBSHGT)
      IF (FROUDE .GT. CRITFN) GO TO 100
C
C     SET UP DRAINAGE VECTOR DIRECTIONS
C
      IF (DELHI .EQ. 0.) THEN
        IF (DELHJ .EQ. 0.) GO TO 100
        IF (DELHJ .LT .0.) THET=270.
        IF (DELHJ .GT. 0.) THET=90.
      ELSE
        THETP=ATAN(DELHJ/DELHI)*DPR
      ENDIF
      IF (DELHI .LT. 0.) THET=THETP+180.
      IF (DELHI .GT. 0.) THEN
        IF (DELHJ .GT. 0.) THEN
          THET=THETP
        ELSE
          THET=THETP+360.
        ENDIF
      ENDIF
      IF (THET .GE. 0. .AND. THET .LE. 90) THETD=90.-THET
      IF (THET .GT. 90. .AND. THET .LE. 360) THETD=450.-THET
C
C     RESOLVE DRAINAGE VECTOR INTO COMPONENTS
C
      ANG=(270.-THETD)*RPD
      DRX=-COS(ANG)
      DRY=-SIN(ANG)
C
C     ADJUST WINDS
C     FIRST NORMALIZE WIND VECTOR
C
      UN=U(I,J,K)/SPEED
      VN=V(I,J,K)/SPEED
C
C     DETERMINE IF WIND IS BLOWING TOWARD OBSTACLE (AA>0)
C
      AA=DRX*UN+DRY*VN
      IF (AA .LE. 0) GO TO 100
C
C     DETERMINE TANGENT WIND VECTOR
C
      TT=-DRY*UN+DRX*VN
      IF (TT .GT. 0.) THEN
        TAX=-DRY
        TAY=DRX
      ELSE
        TAX=DRY
        TAY=-DRX
      ENDIF
      U(I,J,K)=TAX*SPEED
      V(I,J,K)=TAY*SPEED
  100 CONTINUE
C
      RETURN
      END