      SUBROUTINE INTER2(STDAT1,NST,XST,YST,RMIN,RMAX1,RMAX2,RMAX3,
     1 R1,R2,NINTRP,RS,IS,WORK,FIELD1,IFLAG,FGFLD,HTOPO,LNDWTR)
C
C     DIAGNO  VERSION 1.1  LEVEL 900221  INTER2    S.DOUGLAS, SAI
C
      include 'diagno.inc'
C
      COMMON /D4/ EDIT,EDITL,IEDIT,IEDITL
      COMMON /D5/ NX,NY,NZ,DX,DY,DZ(NZMAX)
      COMMON /D6/ IRD,IWR,IFILE
      DIMENSION STDAT1(NZMAX,1), XST(1), YST(1), RS(1),
     1 IS(1), WORK(1), FIELD1(NXMAX,NYMAX,1), JS(10),
     2 FGFLD(NXMAX,NYMAX,1), FIELD2(NXMAX,NYMAX,NZMAX),NINTRP(1),
     3 HTOPO(NXMAX,1), LNDWTR(NXMAX,1)
C
C     COMPUTE 3-D WIND FIELD FROM STATION DATA
C     BY THE ERT INTERPOLATION SCHEME
C
C     INPUTS:  STDAT1 (R ARRAY) - WIND COMPONENT STATION DATA
C              NST (I)          - NUMBER OF STATIONS
C              XST (R ARRAY)    - LOCAL X-COORDINATES OF STATIONS
C              YST (R ARRAY)    - LOCAL Y-COORDINATES OF STATIONS
C              RMIN (R)         - MINIMUM RADIUS OF INFLUENCE
C              RMAX1 (R)        - MAXIMUM RADIUS OF INFLUENCE OVER
C                                 LAND IN THE SURFACE LAYER
C              RMAX2 (R)        - MAXIMUM RADIUS OF INFLUENCE OVER
C                                 LAND IN UPPER LAYERS
C              RMAX3 (R)        - MAXIMUM RADIUS OF INFLUENCE OVER
C                                 WATER
C              R1 (R)           - WEIGHTING PARAMETER FOR SURFACE
C                                 LAYER
C              R2 (R)           - WEIGHTING PARAMETER FOR UPPER
C                                 LAYERS
C            FGFLD (R ARRAY)     - DIAGNOSTIC WIND FIELD
C            HTOPO (R ARRAY)     - GRIDDED TERRAIN HEIGHTS
C            LNDWTR (I ARRAY)    - GRIDDED SURFACE TYPE INDICATORS
C
C     OUTPUTS:  FIELD1 (R ARRAY) - INTERPOLATED WIND COMPONENT
C                                  ARRAY
C
      DATA ZERO/0./
C
C     INITIALIZE FIELD2 ARRAY
C
      N = NXMAX * NYMAX * NZMAX
      CALL XMIT(-N,ZERO,FIELD2)
C
C     CONVERT GRID SPACING TO KM
C
      DXK=DX*0.001
      DYK=DY*0.001

C
C     DETERMINE RMAX
C
      RMAX = AMAX1(RMAX1,RMAX2,RMAX3)
C
      DO 200 J=1,NY
      DO 200 I=1,NX
c      X = (I-0.5)*DXK
c      Y = (J-0.5)*DYK
      X = (I-0.5)*DX
      Y = (J-0.5)*DY
      CALL XMIT(-NST,EDIT,RS)
      DO 120 L=1,NST
      IF(IFLAG.LE.0) GO TO 90
C
C     CHECK FOR BARRIERS TO INTERPOLATION
C
      CALL BARIER(X,Y,XST(L),YST(L),OK)
      IF(OK.LE.0.) GO TO 115
C
C     SKIP STATION IF NO DATA AT ANY ELEVATION
C
   90 DO 100 K=1,NZ
      IF(STDAT1(K,L).LT.EDITL) GO TO 110
  100 CONTINUE
      GO TO 115
C
C     COMPUTE DISTANCE TO STATIONS LESS THAN RMAX AWAY
C
  110 RSX = X - XST(L)
      RSY = Y - YST(L)
      RS(L) = RSX**2 + RSY**2
      RS(L) = SQRT(RS(L))
      IF(RS(L).LT.RMIN) RS(L)=RMIN
      IF(RS(L) .LE. RMAX)  GO TO 115
      RS(L) = EDIT
  115 CONTINUE
  120 CONTINUE
C
C     ORDER STATIONS
C
      DO 121 NN = 1,NST
        IS(NN) = IEDIT
  121 CONTINUE
      CALL XMIT(NST,RS,WORK)
      N = 0
      DO 130 L = 1,NST
      CALL FMINF(WORK,NST,FMIN,NMIN)
      WORK(NMIN) = EDIT + L
      IS(L) = NMIN
      IF(FMIN.GT.EDITL)  GO TO 150
      N = N + 1
  130 CONTINUE
  150 CONTINUE
      DO 190 K=1,NZ
C
C     FIND STATIONS WITH DATA FOR THIS LEVEL
C
      IF (K .EQ. 1 ) THEN
        R = R1
      ELSE
        R = R2
      ENDIF
      NK = 0
      DO 160 M=1,N
      L=IS(M)
      IF(STDAT1(K,L).GT.EDITL)  GO TO 160
      IF (K .EQ. 1) THEN
        IF (RS(L) .GT. RMAX1 .AND. LNDWTR(I,J) .EQ. 1) GO TO 160
      ENDIF
      IF (K .GT. 1) THEN
        IF (RS(L) .GT. RMAX2 .AND. LNDWTR(I,J) .EQ. 1) GO TO 160
      ENDIF
      NK = NK + 1
      JS(NK) = IS(M)
      IF(NK.EQ.NINTRP(K)) GO TO 165
  160 CONTINUE
      IF (NK .GT. 0 .OR. LNDWTR(I,J) .EQ. 1) GO TO 165
c      WRITE(IWR,2065) I,J,K
c      STOP
       call runend(-1,'error in inter2')
  165 CONTINUE
C
C     FIND THE INTERPOLATION WEIGHTING FACTOR FOR EACH VALID STATION
C
      DO 180 M = 1,NK
      L=JS(M)
      IF(STDAT1(K,L) .GT. EDITL)  GO TO 180
      WT = 1./(RS(L)**2)
C
C     APPLY WEIGHTING FACTOR
C
  175 CONTINUE
      FIELD1(I,J,K) = FIELD1(I,J,K) + STDAT1(K,L)*WT
      FIELD2(I,J,K) = FIELD2(I,J,K) + WT
  180 CONTINUE
      IF (LNDWTR(I,J) .EQ. 1) THEN
        FIELD1(I,J,K) = (FIELD1(I,J,K) + FGFLD(I,J,K)/R**2)/
     1                  (FIELD2(I,J,K) + 1./R**2)
      ELSE
        FIELD1(I,J,K) = FIELD1(I,J,K)/FIELD2(I,J,K)
      ENDIF
  190 CONTINUE
  200 CONTINUE
      RETURN
      END