      SUBROUTINE INTERP(STDAT1,NST,XST,YST,RMIN,RMAX,
     1     NINTRP,RS,IS,WORK,FIELD1,IFLAG)
C
C     DIAGNO  VERSION 1.1  LEVEL 900221  INTERP  S.DOUGLAS, SAI
C
      include 'diagno.inc'
C
      COMMON /D4/ EDIT,EDITL,IEDIT,IEDITL
      COMMON /D5/ NX,NY,NZ,DX,DY,DZ(NZMAX)
      COMMON /D6/ IRD,IWR,IFILE
      DIMENSION STDAT1(NZMAX,1), XST(1), YST(1), RS(1),
     1 IS(1), WORK(1), FIELD1(NXMAX,NYMAX,1), JS(10), NINTRP(5)
C
C
C       COMPUTES 3-D WIND FIELD FROM STATION DATA
C       BY THE ERT INTERPOLATION SCHEME
C
C     INPUTS:  STDAT1 (R ARRAY) - WIND COMPONENT STATION DATA
C              NST (I)          - NUMBER OF STATIONS
C              XST (R ARRAY)    - LOCAL X-COORDINATES OF STATIONS
C              YST (R ARRAY)    - LOCAL Y-COORDINATES OF STATIONS
C              RMIN (R)         - MINIMUM RADIUS OF INFLUENCE
C              RMAX (R)         - MAXIMUM RADIUS OF INFLUENCE
C              NINTRP(I)        - NO. OF STATIONS USED IN THE
C                                 INTERPOLATION TO A GRID POINT
C              IFLAG (I)        - NO. OF BARRIERS
C
C     OUTPUTS:  FIELD1 (R ARRAY) - INTERPOLATED WIND COMPONENT ARRAY
C
C     CONVERT GRID SPACING TO KM
C
      DXK=DX*0.001
      DYK=DY*0.001
C
      DO 200 J=1,NY
      DO 200 I=1,NX
      X = (I-0.5)*DXK
      Y = (J-0.5)*DYK
      CALL XMIT(-NST,EDIT,RS)
      DO 120 L=1,NST
      IF(IFLAG.LE.0) GO TO 90
C
C     CHECK FOR BARRIERS TO INTERPOLATION
C
      CALL BARIER(X,Y,XST(L),YST(L),OK)
      IF(OK.LE.0.) GO TO 115
C
C     SKIP STATION IF NO DATA AT ANY ELEVATION
C
   90 DO 100 K=1,NZ
      IF(STDAT1(K,L).LT.EDITL) GO TO 110
  100 CONTINUE
      GO TO 115
C
C     COMPUTE DISTANCE TO STATIONS LESS THAN RMAX AWAY
C
  110 RSX = X - XST(L)
      RSY = Y - YST(L)
      RS(L) = RSX**2 + RSY**2
      RS(L) = SQRT(RS(L))
      IF(RS(L).LT.RMIN) RS(L)=RMIN
      IF(RS(L) .LE. RMAX)  GO TO 115
      RS(L) = EDIT
  115 CONTINUE
  120 CONTINUE
C
C     ORDER STATIONS
C
      DO 121 NN = 1,NST
      IS(NN)=IEDIT
  121 CONTINUE
      CALL XMIT(NST,RS,WORK)
      N = 0
      DO 130 L = 1,NST
      CALL FMINF(WORK,NST,FMIN,NMIN)
      WORK(NMIN) = EDIT + L
      IS(L) = NMIN
      IF(FMIN.GT.EDITL)  GO TO 140
      N = N + 1
  130 CONTINUE
  140 IF(N.GT.0)  GO TO 150
c  145 WRITE(IWR,2064) I,J
c      STOP
      call runend(-1,'error in interp')
  150 CONTINUE
      DO 190 K=1,NZ
C
C     FIND SUM OF DISTANCES TO STATIONS WITH DATA FOR THIS LEVEL
C
      RSUM = 0.
      NK = 0
      DO 160 M=1,N
      L=IS(M)
      IF(STDAT1(K,L).GT.EDITL)  GO TO 160
  153 IF(K.GT.1)  GO TO 155
      IF (RS(L) .GT. RMAX) GO TO 160
      RSUM = RSUM + 1./RS(L)**2
      GO TO 158
  155 RSUM = RSUM + 1./RS(L)
  158 CONTINUE
      NK = NK + 1
      JS(NK) = IS(M)
      IF(NK .EQ. NINTRP(K)) GO TO 165
  160 CONTINUE
      IF(NK .GT. 0) GO TO 165
c      WRITE(IWR,2065) I,J,K
c      STOP
      call runend(-1,'error in interp')
  165 CONTINUE
C
C     FIND THE INTERPOLATION WEIGHTING FACTOR FOR EACH VALID STATION
C
      DO 180 M = 1,NK
      L=JS(M)
      IF(STDAT1(K,L) .GT. EDITL)  GO TO 180
      IF(K.GT.1)  GO TO 170
      WT = 1./((RS(L)**2)*RSUM)
      GO TO 175
  170 WT = 1./(RS(L)*RSUM)
C
C     APPLY WEIGHTING FACTOR
C
  175 CONTINUE
      FIELD1(I,J,K) = FIELD1(I,J,K) + STDAT1(K,L)*WT
  180 CONTINUE
  190 CONTINUE
  200 CONTINUE
      RETURN
      END