      SUBROUTINE MINIM(U,V,W,UB,VB,DIV,NITER,DIVLIM)
C
C     DIAGNO  VERSION 1.1  LEVEL 900221  MINIM    S.DOUGLAS, SAI
C
      include 'diagno.inc'
C
      DIMENSION U(NXMAX,NYMAX,1),V(NXMAX,NYMAX,1),W(NXMAX,NYMAX,1),
     1          UB(NYMAX,2,1),VB(NXMAX,2,1),DIV(NXMAX,NYMAX,1)
      COMMON /D5/ NX,NY,NZ,DX,DY,DZ(NZMAX)
      COMMON /D6/ IRD,IWR,IFILE
C
C     ITERATIVE SCHEME TO MINIMIZE DIVERGENCE
C
C     INPUTS:  U (R ARRAY)   - GRIDDED X-DIRECTION WIND COMPONENTS
C              V (R ARRAY)   - GRIDDED Y-DIRECTION WIND COMPONENTS
C              W (R ARRAY)   - GRIDDED VERTICAL WIND COMPONENTS
C              UB (R ARRAY)  - U-COMPONENT BOUNDARY VALUES
C              VB (R ARRAY)  - V-COMPONENT BOUNDARY VALUES
C              DIV (R ARRAY) - 3-D DIVERGENCE
C              NITER (I)     - MAXIMUM NUMBER OF ITERATIONS
C              DIVLIM (R)    - MAXIMUM DIVERGENCE
C
C
C     OUTPUTS:  U (R ARRAY) - NON-DIVERGENT X-DIRECTION WIND
C                             COMPONENT
C               V (R ARRAY) - NON-DIVERGENT Y-DIRECTION WIND
C                             COMPONENT
C      WRITE(IWR,2809)
      DO 10 K=1,NZ
      ITER=0
C
C     COMPUTE DIVERGENCE
C
      DIVMAX=-1.0E+09
      CALL DIVCEL(U,V,W,DIV,UB,VB,DIVMAX,K)
      IF (DIVMAX .LE. DIVLIM) GO TO 10
C
C     ADJUST HORIZONTAL WIND FIELDS
C
      DO 20 ITER=1,NITER
      DO 30 IDIR=1,4
      DO 40 JJ=1,NY
      DO 40 II=1,NX
      GO TO (50,60,70,80), IDIR
  50  I=II
      J=JJ
      GO TO 90
  60  I=NX-II+1
      J=JJ
      GO TO 90
  70  I=II
      J=NY-JJ+1
      GO TO 90
  80  I=NX-II+1
      J=NY-JJ+1
  90  CONTINUE
      IF (DIV(I,J,K) .EQ. 0.) GO TO 40
      IP1=I+1
      IM1=I-1
      JP1=J+1
      JM1=J-1
      UIM1=UB(J,1,K)
      IF (I .GT. 1) UIM1=U(IM1,J,K)
      VJM1=VB(I,1,K)
      IF (J .GT. 1) VJM1=V(I,JM1,K)
      UIP1=UB(J,2,K)
      IF (I .LT. NX) UIP1=U(IP1,J,K)
      VJP1=VB(I,2,K)
      IF (J .LT. NY) VJP1=V(I,JP1,K)
      ALPHA1=0.5
      ALPHA2=0.5
      ALPHA3=0.5
      ALPHA4=0.5
      AL1234=ALPHA1+ALPHA2+ALPHA3+ALPHA4
      IF (AL1234 .LT. 1.E-6) GO TO 40
      UT=-2.*(DIV(I,J,K)*DX)/AL1234
      VT=-2.*(DIV(I,J,K)*DY)/AL1234
      UIP1=UIP1+ALPHA1*UT
      UIM1=UIM1-ALPHA2*UT
      VJP1=VJP1+ALPHA3*VT
      VJM1=VJM1-ALPHA4*VT
      IF (I .GT. 1) U(IM1,J,K)=UIM1
      IF (I .LT. NX) U(IP1,J,K)=UIP1
      IF (J .GT. 1) V(I,JM1,K)=VJM1
      IF (J .LT. NY) V(I,JP1,K)=VJP1
  40  CONTINUE
      DIVMAX=-1.0E+09
      CALL DIVCEL(U,V,W,DIV,UB,VB,DIVMAX,K)
C
C     RESET BOUNDARY CONDITIONS
C
      CALL WINDBC(U,V,UB,VB,K)
   30 CONTINUE
C
C     CONVERGENCE TEST FOR DIVERGENCE MAGNITUDE
C
      IF (DIVMAX .LE. DIVLIM) GO TO 10
   20 CONTINUE
c   10 WRITE(IWR,2810) K,ITER,DIVMAX
   10 CONTINUE
C
 2809 FORMAT(//,5X,'SUMMARY OF DIVERGENCE MINIMIZATION'
     1,//,'  LEVEL   ITERATIONS   MAXIMUM DIVERGENCE (/SEC)')
 2810 FORMAT(3X,I2,I11,12X,E10.3)
      RETURN
      END