	 subroutine readgrd
!****************************************************************
!*
!*    Reads GRID BLOCK from the input file and allocates memory
!*
!****************************************************************
     use io_parameters
	 use master_variables
	 implicit none
!
	 character(len=100) :: message
     integer  (kind=4)  :: istat
!
!***  GRID BLOCK (except topography-related variables)
!
     call get_input_int4(finp,'GRID','NX',nx,1,istat,message)
     if(istat.ne.0) call runend(-1,message)
!
     call get_input_int4(finp,'GRID','NY',ny,1,istat,message)
     if(istat.ne.0) call runend(-1,message)
!
     call get_input_rea4(finp,'GRID','DX_(M)',dx,1,istat,message)
     if(istat.ne.0) call runend(-1,message)
!
     call get_input_rea4(finp,'GRID','DY_(M)',dy,1,istat,message)
     if(istat.ne.0) call runend(-1,message)
!
     call get_input_rea4(finp,'GRID','X_ORIGIN_(UTM_M)',x0,1,istat,message)
     if(istat.ne.0) call runend(-1,message)
!
     call get_input_rea4(finp,'GRID','Y_ORIGIN_(UTM_M)',y0,1,istat,message)
     if(istat.ne.0) call runend(-1,message)
!
!*** Allocates memory
!
     call getmem
!
!*** Creates (or reads from a regional file) the topography 
!
     call readtop 
!
     return
     end subroutine readgrd
!
!
!
	 subroutine readtop
!****************************************************************
!*
!*    Reads/creates the topography 
!*
!****************************************************************
     use io_parameters
	 use master_variables
     implicit none
!
	 character(len=100) :: message
	 character(len=10)  :: word
     integer  (kind=4)  :: istat,ix,iy
     real     (kind=4)  :: thx0,thy0,z0
!
!*** Reads flag
!
     call get_input_cha (finp,'GRID','EXTRACT_TOPOGRAPHY_FROM_FILE',word,1,istat,message)
	 if(istat.ne.0) call runend(-1,message)
!
	 if(TRIM(word).eq.'YES'.or.TRIM(word).eq.'yes') then
!
!***    Read topography from a file 
!
        call readem(ftop,x0,y0,dx,dy,nx,ny,g)
!
     else 
!
!*** Creates topography
!
!
        call get_input_rea4(finp,'GRID','Z_ORIGIN_(M)',z0,1,istat,message)
        if(istat.ne.0) call runend(-1,message)
!
        call get_input_rea4(finp,'GRID','X_SLOPE_(DEG)',thx0,1,istat,message)
        if(istat.ne.0) call runend(-1,message) 
		thx0 = tan(thx0*pi/180.)                 ! Convert from deg to rad
!
        call get_input_rea4(finp,'GRID','Y_SLOPE_(DEG)',thy0,1,istat,message)
        if(istat.ne.0) call runend(-1,message)
		thy0 = tan(thy0*pi/180.)                 ! Convert from deg to rad
!
        do iy=1,ny
		   do ix = 1,nx
			  g(ix,iy) = z0 + thx0*(ix-1)*dx + thy0*(iy-1)*dy
		   end do
		end do
!
	 end if      
!
     return
	 end subroutine readtop
!
!
!
     subroutine readem(fdem,xo,yo,dx,dy,nx,ny,g)
!************************************************************
!*
!*   Reads terrain data from a surfer GRD ASCII file 
!*
!************************************************************
     implicit none 
!
     character(len=250) :: fdem
     integer  (kind=4 ) :: nx,ny,ix,iy
	 real     (kind=4 ) :: xo,yo,xf,yf,dx,dy,x,y
	 real     (kind=4 ) :: g(nx,ny)
!
     integer  (kind=4 ) :: nxdem,nydem,ixdem,iydem
	 real     (kind=4 ) :: xodem,yodem,xfdem,yfdem,dxdem,dydem
     real     (kind=4 ) :: get_val
!
     real     (kind=4), allocatable :: dem(:,:)   
!
!*** Comutes xf and yf
!
     xf = xo + (nx-1)*dx
     yf = yo + (ny-1)*dy
!
!*** Reads the file
!
     open(90,FILE=TRIM(fdem),STATUS='old',err=100)
     read(90,*,err=101) nxdem,nydem
	 read(90,*,err=101) xodem,xfdem
	 read(90,*,err=101) yodem,yfdem
!
!*** Checks that the computational domain is within the regional DEM file
!
     if(xodem.gt.xo) call runend(-1,'x0 of the domain is outside the DEM file')
     if(xfdem.lt.xf) call runend(-1,'xf of the domain is outside the DEM file')
     if(yodem.gt.yo) call runend(-1,'yo of the domain is outside the DEM file')
     if(yfdem.lt.yf) call runend(-1,'yf of the domain is outside the DEM file')
!     
	 allocate(dem(nxdem,nydem))
	 dxdem = (xfdem-xodem)/(nxdem-1)
	 dydem = (yfdem-yodem)/(nydem-1)
     do iydem=1,nydem
	    read(90,*,err=101) (dem(ixdem,iydem),ixdem=1,nxdem) 
	 end do
     close(90)
!
!*** Interpolates 
!
      do iy = 1,ny
         y = yo + (iy-1)*dy
         do ix = 1,nx
            x = xo + (ix-1)*dx
	        g(ix,iy) = get_val(xodem,yodem,nxdem,nydem,dxdem,dydem,dem,x,y)
         end do
      end do
!
!*** Releases memory
!
     deallocate(dem)
     return
!
!*** List of errors
!
 100 call runend(-1,'Error opening the DEM file'//TRIM(fdem))
 101 call runend(-1,'Error reading the DEM file'//TRIM(fdem))
     return
     end subroutine readem
!
!
!
     real(kind=4) function get_val(xodem,yodem,nxdem,nydem,dxdem,dydem,dem,x,y)
!***********************************************************
!*
!*   Interpolates dem at the (x,y) point
!*
!***********************************************************
     implicit none
!
     integer(kind=4) :: nxdem,nydem
     real   (kind=4) :: xodem,yodem,dxdem,dydem,dem(nxdem,nydem)
	 real   (kind=4) :: x,y,xdem,ydem
!
     logical         :: xfound,yfound
     integer(kind=4) :: iydem,ixdem,ixpt,iypt
     real   (kind=4) :: s,t,st,shape(4)
!
     xfound = .false.
     yfound = .false.
!
     do ixdem = 1,nxdem-1
        xdem = xodem + (ixdem-1)*dxdem
		if((x.ge.xdem).and.(x.le.(xdem+dxdem))) then
           ixpt   = ixdem
	       xfound = .true.
	    end if
	 end do  	   
     if(.not.xfound) call runend(-1,'get_val: x value not found')
!
     do iydem = 1,nydem-1
        ydem = yodem + (iydem-1)*dydem
        if((y.ge.ydem).and.(y.le.(ydem+dydem))) then
           iypt   = iydem
	       yfound = .true.
	    end if
	 end do 
     if(.not.yfound) call runend(-1,'get_val: y value not found')
!
!*** Interpolates
!
     s = (x-(xodem+(ixpt-1)*dxdem))/dxdem      ! parameter s in (0,1)
     s = 2.*s-1.                               ! parameter s in (-1,1)
     t = (y-(yodem+(iypt-1)*dydem))/dydem      ! parameter t in (0,1)
     t = 2.*t-1.                               ! parameter t in (-1,1)
	 st = s*t
!     
     shape(1)=(1.-t-s+st)*0.25                      ! 4         3
     shape(2)=(1.-t+s-st)*0.25                      !
     shape(3)=(1.+t+s+st)*0.25                      !      
     shape(4)=(1.+t-s-st)*0.25                      ! 1         2
!
     get_val = shape(1)*dem(ixpt  ,iypt  ) +  &
               shape(2)*dem(ixpt+1,iypt  ) +  &
               shape(3)*dem(ixpt+1,iypt+1) +  &
               shape(4)*dem(ixpt  ,iypt+1) 
!
	 return
	 end function get_val


     
