     subroutine readzrou
!************************************************************
!*
!*   Reads zroughness from a surfer GRD ASCII file 
!*
!************************************************************
     use master_variables
	 use io_parameters
     implicit none 
!
     integer  (kind=4 ) :: nxdem,nydem,ixdem,iydem,iy,ix
	 real     (kind=4 ) :: xodem,yodem,xfdem,yfdem,dxdem,dydem,x,y
     real     (kind=4 ) :: get_val
!
     real     (kind=4), allocatable :: dem(:,:)   
!
!*** Reads the file
!
     open(90,FILE=TRIM(frou),STATUS='old',err=100)
     read(90,*,err=101) nxdem,nydem
	 read(90,*,err=101) xodem,xfdem
	 read(90,*,err=101) yodem,yfdem
!
!*** Checks that the station is within the regional roughness file
!
     if(xodem.gt.x_stat) call runend(-1,'x_sta is outside the roughness file')
     if(xfdem.lt.x_stat) call runend(-1,'x_sta is outside the roughness file')
     if(yodem.gt.y_stat) call runend(-1,'y_sta is outside the roughness file')
     if(yfdem.lt.y_stat) call runend(-1,'y_sta is outside the roughness file')
!
!*** Checks that the computational domain is within the regional roughness file
!
     if(xodem.gt.x0              ) call runend(-1,'x0 of the domain is outside the rough file')
     if(xfdem.lt.(x0 + (nx-1)*dx)) call runend(-1,'xf of the domain is outside the rough file')
     if(yodem.gt.y0              ) call runend(-1,'yo of the domain is outside the rough file')
     if(yfdem.lt.(y0 + (ny-1)*dy)) call runend(-1,'yf of the domain is outside the rough file')
!     
	 allocate(dem(nxdem,nydem))
	 dxdem = (xfdem-xodem)/(nxdem-1)
	 dydem = (yfdem-yodem)/(nydem-1)
     do iydem=1,nydem
	    read(90,*,err=101) (dem(ixdem,iydem),ixdem=1,nxdem) 
	 end do
     close(90)
!
!*** Interpolates to get zrough
!
     do iy = 1,ny 
	    y = y0 + (iy-1)*dy
	    do ix = 1,nx
		   x = x0 + (ix-1)*dx
           zrough(ix,iy) = get_val(xodem,yodem,nxdem,nydem,dxdem,dydem,dem,x,y)
        end do
	 end do
!
     zrough_stat = get_val(xodem,yodem,nxdem,nydem,dxdem,dydem,dem,x_stat,y_stat)
!
!*** Releases memory
!
     deallocate(dem)
     return
!
!*** List of errors
!
 100 call runend(-1,'Error opening the terrain file'//TRIM(frou))
 101 call runend(-1,'Error reading the terrain file'//TRIM(frou))
     return
     end subroutine readzrou