      subroutine RUN_DIAGNO
C
C     DIAGNO VERSION 1.1   LEVEL 900221   R. KESSLER, S. DOUGLAS, SAI
C
C     DIAGNO IS A DIAGNOSTIC WIND MODULE WHICH GENERATES A THREE-
C     DIMENSIONAL, NON-DIVERGENT WIND FIELD IN TERRAIN FOLLOWING
C     COORDINATES.  BEGINNING WITH A DOMAIN-SCALE MEAN WIND, THE
C     VERTICAL VELOCITY DUE TO TOPOGRAPHIC EFFECTS IS CALCULATED.
C     THE HORIZONTAL VELOCITY FIELD IS THEN ADJUSTED TO MINIMIZE
C     THE DIVERGENCE SUBJECT TO THE CONSTRAINT THAT THE VERTICAL
C     VELOCITY IS HELD CONSTANT.  THERMALLY-INDUCED SLOPE FLOWS
C     ARE GENERATED AND ADDED TO THE WIND FIELD.  A FROUDE NUMBER
C     ADJUSTMENT PROCEDURE IS USED TO DIVERT THE FLOW AROUND
C     TERRAIN OBSTACLES.  THE FIRST-GUESS FIELD IS COMBINED WITH
C     OBSERVATIONS TO PRODUCE A GRIDDED WIND FIELD.  FINALLY A NEW
C     VERTICAL VELOCITY IS CALCULATED USING THE CONTINUITY EQUATION
C     AND IS ADJUSTED SO THAT THE VERTICAL VELOCITY AT THE TOP OF
C     THE MODEL IS ZERO.  THE DIVERGENCE IS MINIMIZED SUBJECT TO
C     THE CONSTRAINT THAT THE VERTICAL VELOCITY IS UNCHANGED.
C
C     THE DIAGNOSTIC WIND MODULE IS AN ENHANCED VERSION OF THE CIT
C     WIND MODEL (GOODIN,MCRAE AND SEINFELD, 1980).  THE FROUDE
C     NUMBER MODIFICATION SCHEME IS ADAPTED FROM THE MELSAR MODEL
C     (ALLWINE AND WHITEMAN, 1985).  THE KINEMATIC AND SLOPE FLOW
C     PARAMETERIZATIONS ARE DERIVED FORM THE COMPLEX-TERRAIN WIND
C     MODEL (YOCKE, 1979).
C
      include 'diagno.inc'

      CHARACTER*4 NAMST(NWINDM),LAST,IBLANK,NAM
      CHARACTER*20 TITLE
	LOGICAL go_on
C
      DIMENSION U(NXMAX,NYMAX,NZMAX),V(NXMAX,NYMAX,NZMAX),
     1  W(NXMAX,NYMAX,NZPMAX), UB(NYMAX,2,NZMAX), VB(NXMAX,2,NZMAX),
     2  USLOPE(NXMAX,NYMAX,NZMAX),  VSLOPE(NXMAX,NYMAX,NZMAX),
     3  UG(NXMAX,NYMAX,NZMAX),  VG(NXMAX,NYMAX,NZMAX),
     4  HTOPO(NXMAX,NYMAX), BETA2(24), HMAX(NXMAX,NYMAX),
     5  UTMXST(NWINDM), UTMYST(NWINDM), WT(NWINDM),
     6  RS(NWINDM), IS(NWINDM), IST(NWINDM), JST(NWINDM),
     7  US(NZMAX,NWINDM), VS(NZMAX,NWINDM),
     8  CELLZB(NZPMAX), CELLZC(NZMAX),
     9  FEXTRP(NZMAX), DIV(NXMAX,NYMAX,NZMAX),
     1  UM(24), VM(24), NINTRP(NZMAX), LNDWTR(NXMAX,NYMAX),
     2  ALW(NXMAX,NYMAX)
      COMMON /D2/ WORK(MAX), WORK2(3)
      COMMON /D3/ IFIN, NUMBAR, BARXY(4,MAXBAR), SLPIN(2,MAXBAR)
      COMMON /D4/ EDIT, EDITL, IEDIT, IEDITL
      COMMON /D5/  NX, NY, NZ, DX, DY, DZ(NZMAX)
      COMMON /D6/  IRD, IWR, IFILE
      COMMON /TEMPE/TZ00,TZ11,Z00,Z11,GAMMA(24)
C
      DATA ZERO /0./, IZERO /0/, LAST /'LAST'/, IBLANK /'    '/, IONE/1/
      DATA RAD /57.29577951/
C
C     SET EDIT FLAGS
C
      EDIT = 999.
      EDITL= 900.
      IEDIT = 999
      IEDITL = 900
C
C     SET LOGICAL UNITS
C
      IRD = 12
      IRD1 = 7
      IRD2 = 8
      IRDT = 5
      IRDL = 4
      IWR = 6
      IFILE = 9
      IFILEK = 11
      IFILEF = 13
      IFILES = 14
C
C     READ INITIAL INPUT DATA
C
      CALL SETUP(NX,NY,NZ,DXK,DYK,CELLZB,UTMXOR,UTMYOR,
     1    TSTART,TEND,NWIND,NUPPER,ZSWIND,
     2    RMIN,RMAX1,RMAX2,RMAX3,R1,R2,NINTRP,
     3    NZPRNT,IPR0,IPR1,IPR2,IPR3,IPR4,IPR5,
     4    IPR6,IPR7,ICALC,IOUTD,HTFAC,NITER,DIVLIM,
     5    IOBR,NUMBAR,I3DCTW,NSMTH,IEXTRP,FEXTRP,
     6    Z00,Z11,TZ00,TZ11)
C
      IF (I3DCTW .EQ. 1) CALL PARM3D(CRITFN,TERRAD,
     1                   TINF,BETA2,IFRADJ,IKINE,ALPHA,UM,VM,DX,DY)
C
C     INITIALIZE SOME PARAMETERS
C
      IFIN = 0
      NZP1=NZ+1
      NSURF=NWIND-NUPPER      
c      NOHRS = INT((TEND - TSTART)/3600.) + 1
C
C     CONVERT FROM KM
C      
	DX = DXK*1000.
      DY = DYK*1000.
C
C     CHECK THAT GRID DIMENSIONS DO NOT EXCEED MAXIMUM DIMENSIONS
C
      IF(NX.GT.NXMAX  .OR. NY.GT.NYMAX  .OR.  NZ.GT.NZMAX) 
     .   call runend(-1,'Grid dimensions larger than maximum') 
C
C     CALCULATE CELL DEPTHS AND CELL CENTER HEIGHTS
C     (DZ IS VERTICAL CELL DEPTH; CELLZC IS VERTICAL CELL CENTER
C      ELEVATION)
C
    3 DO 1 I=1,NZ
      DZ(I) = CELLZB(I+1)  -  CELLZB(I)
      CELLZC(I) = CELLZB(I) + .5*DZ(I)
    1 CONTINUE
C
C     INITIALIZE TERRAIN ARRAYS TO ZERO
C
      N = NXMAX*NYMAX
      CALL XMIT(-N,ZERO,HTOPO)
      CALL XMIT(-N,ZERO,HMAX)
      CALL IXMIT(-N,IZERO,LNDWTR)
C
C     READ TERRAIN AND LAND USE DATA
C     HTOPO IS HEIGHT OF TOPOGRAPHY (UNITS OF HTFAC*METERS), FOR
C     EACH GRID SQUARE, ABOVE A BASE ELEVATION (E.G.,MSL)
C
      call get_HTOPO(HTOPO,LNDWTR,NXMAX,NYMAX)
C
C     CONVERT TERRAIN HEIGHT TO METERS
C
      DO 7 J=1,NY
      DO 7 I=1,NX
      HTOPO(I,J)=HTOPO(I,J)*HTFAC
      HTMIN=AMIN1(HTOPO(I,J),HTMIN)
      HTMAX=AMAX1(HTOPO(I,J),HTMAX)
    7 CONTINUE
C
C     SET UP THE HMAX ARRAY (TERRAD=1.)
C     HMAX IS THE MAXIMUM TERRAIN HEIGHT WITHIN A GIVEN RADIUS
C
      IF (I3DCTW .EQ. 1) CALL TERSET(HTOPO,HMAX,TERRAD)
C
C     CREATES A NUMBER MAP OF THE ARRAY HTOPO and HMAX
C
      CALL WNDLPT(HTOPO)
      RANGE=HTMAX-HTMIN
      CALL WNDLPT(HMAX)
C
C     OUTPUT LNDWTR TYPE DATA
C
      DO 1811 J = 1,NY
      DO 1811 I = 1,NX
        ALW(I,J) = LNDWTR(I,J)
 1811 CONTINUE
      CALL WNDLPT(ALW)
C
C     FOR THE CASE OF NO OBSERVATIONS, PROCEED DIRECTLY TO THE
C     CALCULATION OF THE DIAGNOSTIC WIND FIELD
C
      IF (NWIND .EQ. 0) GO TO 27
C
C     READ WIND STATION NAMES AND UTM COORDINATES (IN KM)
C
      IF(NWIND.GT.NWINDM)  
     . call runend(-1,'Too many stations. Change NWINDM')
      IF(NUPPER.GE.NUPPRM) 
     . call runend(-1,'Too many upper stations. Change NUPPRM')
C
      call get_STAT(NAMST,UTMXST,UTMYST)
C
C     CONVERT TO LOCAL COORDINATES
C
      DO 19 I=1,NWIND
        UTMXST(I) = UTMXST(I) - UTMXOR    
        UTMYST(I) = UTMYST(I) - UTMYOR    
        IST(I) = UTMXST(I)/DXK + 1
        JST(I) = UTMYST(I)/DYK + 1
   19 CONTINUE
C
C     BEGIN SIMULATION                                          .
C
  27  TIME = TSTART
c
      go_on = .true.
	do while(go_on)
c      DO 900 NH=1,NOHRS
      IT = 1
      IF (I3DCTW .NE. 1) GO TO 292
C
C     READ SURFACE WIND DATA
C
      call readuni(time,TIME2,WT(1),US(1,1),VS(1,1),TZ00,TZ11)
c
c     update values of tempe related variables
c
      IF (I3DCTW .EQ. 1) CALL PARM3D(CRITFN,TERRAD,
     1                   TINF,BETA2,IFRADJ,IKINE,ALPHA,UM,VM,DX,DY)
C
C     SET UP FIRST GUESS FIELD FROM STATION !!
C
      DO 400 K = 1,NZ
      DO 400 J = 1,NYMAX
      DO 400 I = 1,NXMAX
c        UG(I,J,K) = UM(IT)
c        VG(I,J,K) = VM(IT)
        UG(I,J,K) = US(1,1)
        VG(I,J,K) = VS(1,1)
  400 CONTINUE
C
C     SET BOUNDARY CONDITIONS
C
      DO 152 K=1,NZ
      CALL WINDBC(UG,VG,UB,VB,K)
  152 CONTINUE
C
C     INITIALIZE THE VERTICAL VELOCITY VECTOR W
C
      N = NXMAX*NYMAX*NZPMAX
      CALL XMIT(-N,ZERO,W)
C
C     CALCULATE THE VERTICAL VELOCITY DUE TO TOPOGRAPHIC EFFECTS
C
      IF (IKINE .EQ. 1) CALL TOPOF2(UG,VG,W,HTOPO,
     1    CELLZB,GAMMA,TINF,ALPHA,TIME)
C
C     GENERATE A NEW HORIZONTAL VELOCITY FIELD USING THE
C     DIVERGENCE MINIMIZATION PROCEDURE
C
      IF (IKINE .EQ. 1) CALL MINIM(UG,VG,W,UB,VB,DIV,
     1     NITER,DIVLIM)
C
C     OUTPUT KINEMATIC EFFECTS
C
c      IF (IPR5 .LE. 0) GO TO 286
c      CALL WINDPR(UG,VG,W,NZPRNT)
c      IF (IOUTD .GT. 0) CALL OUTFIL(IFILEK,TIME,UG,VG,W)
c  286 CONTINUE

C
C     CALCUATE THE SLOPE FLOW
C
      CALL SLOPE(USLOPE,VSLOPE,HTOPO,BETA2,TINF,TIME,
     1           HMAX,GAMMA)
C
C     ADD THE SLOPE FLOW COMPONENT TO THE HORIZONTAL WIND FIELD
C
      DO 300 K=1,NZ
      DO 300 J=1,NY
      DO 300 I=1,NX
        UG(I,J,K)=UG(I,J,K)+USLOPE(I,J,K)
        VG(I,J,K)=VG(I,J,K)+VSLOPE(I,J,K)
  300 CONTINUE
C
C     OUTPUT SLOPE FLOW EFFECTS
C
C      IF (IPR6 .LE. 0) GO TO 288
C      CALL WNDPR2(UG,VG,NZPRNT)
C      IF (IOUTD .GT. 0) CALL OUTFIL(IFILES,TIME,UG,VG,W)
C  288 CONTINUE

C
C     APPLY THE MELSAR FROUDE NUMBER ADJUSTMENT
C
      IF (IFRADJ .EQ. 1) CALL FRADJ(UG,VG,HTOPO,GAMMA,TINF,CRITFN,
     1                   CELLZB,HMAX,TIME)
C
C     OUTPUT FROUDE NUMBER EFFECTS
C
c      IF (IPR7 .LE. 0) GO TO 287
c      CALL WNDPR2(UG,VG,NZPRNT)
c      IF (IOUTD .GT. 0) CALL OUTFIL(IFILEF,TIME,UG,VG,W)
c  287 CONTINUE

C
C     SET BOUNDARY CONDITIONS
C
      DO 290 K=1,NZ
      CALL WINDBC(UG,VG,UB,VB,K)
  290 CONTINUE
C
C     IN THE CASE OF NO OBSERVATIONS, PROCEED DIRECTLY TO THE
C     SMOOTHING STEP
C
      IF (NWIND .EQ. 0) THEN
        DO 291 K = 1,NZ
        DO 291 J = 1,NY
        DO 291 I = 1,NX
          U(I,J,K) = UG(I,J,K)
          V(I,J,K) = VG(I,J,K)
  291   CONTINUE
      ENDIF
      IF (NWIND .EQ. 0) GO TO 285
C
C     READ HOURLY INPUT DATA
C
  292 CONTINUE
C
C     READ SURFACE WIND DATA
C     (we do it before, to be used as a first guess)

C
C     PRINT INPUT SURFACE WIND DATA
C
c      IF(IPR0.GT.0)  WRITE(IWR,2000) TITLE
c      DO 40 I=1,NSURF
c        WRITE(IWR,2290) NAMST(I),US(1,I),VS(1,I),WT(I)
c   40 CONTINUE
C
C     INPUT UPPER AIR WIND DATA
C     (WINDS ARE INPUT FOR THE CELL CENTER ELEVATION)
C
c   50 IF(NUPPER.EQ.0) GO TO 52
c      N = NSURF + 1
c      CALL XMIT(-NUPPRM,ZERO,WT(N))
c      N = 2*NZ + 1
c   51 READ(IRD2,1041) NAM, (WORK(I),I=1,N)
c      IF(NAM.EQ.LAST)  GO TO 56
c      N1 = NSURF + 1
c      DO 53 J=N1,NWIND
c        IF(NAM.EQ.NAMST(J))  GO TO 54
c   53 CONTINUE
c      WRITE(IWR,2390) TIME,NAM
c      GO TO 1000
c   54 WT(J) = WORK(1)
c      I1=1
C
C     IF A SURFACE STATION ALSO EXISTS AT THIS POINT,
C     DO NOT USE UPPER AIR DATA IN LOWEST LAYER
C
c      DO 57 I = 1,NSURF
c        IF (NAM .EQ. NAMST(I)) I1 = 2
c   57 CONTINUE
c      IF (IEXTRP .LE. 0) I1=2
c      DO 55 I=I1,NZ
c        K = 2*(I-1) + 2
c        US(I,J) = WORK(K)
c        VS(I,J) = WORK(K+1)
c   55 CONTINUE
c      GO TO 51
c   56 CONTINUE
C
C     PRINT INPUT UPPER AIR WIND DATA
C
c      IF(NWIND.GT.30) WRITE(IWR,2000) TITLE
c      NSET = NZ/5
c      IF (MOD(NZ,5) .NE. 0) NSET = NSET + 1
c      DO 59 NS = 1,NSET
c      N1 = (NS - 1) * 5 + 1
c      N2 = NS * 5
c      N2 = MIN0(N2,NZ)
c      WRITE(IWR,2292) (I,I=N1,N2)
c      WRITE(IWR,2293)
c      DO 58 L=1,NUPPER
c        LST = NSURF + L
c        WRITE(IWR,2294) NAMST(LST),(US(I,LST),VS(I,LST),I=N1,N2)
c   58 CONTINUE
c   59 CONTINUE
 
 
   52 IF(ICALC.LE.0)  GO TO 850
C
C     EXTRAPOLATE SURFACE WINDS
C     EXTRAPOLATION OPTIONS:
C       1) IF IABS(IEXTRP)=1, THEN DO NOT EXTRAPOLATE FROM SURFACE DATA
C       2) IF IABS(IEXTRP)=2, THEN USE POWER LAW
C       3) IF IABS(IEXTRP)=3, THEN USE FEXTRP MULTIPLIER
C       4) IF IEXTRP<=0, THEN DO NOT USE LEVEL 1 DATA FROM UA WINDS
C
      IF(IABS(IEXTRP).EQ.1) GO TO 91
      RMIN2 = 4.
      DO 70 L=1,NSURF
      IF(NUPPER.EQ.0) GO TO 63
C
C     CHECK FOR CLOSE UPPER AIR STATION WITH VALID WIND DATA
C     SKIP EXTRAPOLATION IF CLOSE UPPER AIR DATA EXISTS
C
      N = NSURF + 1
      CALL XMIT(-NUPPER,EDIT,RS(N))
C
C     CHECK IF UPPER AIR STATION HAS SOME VALID DATA
C
      DO 64 I = 1,NUPPER
      DO 61 K = 1,NZ
        IF(US(K,L).LT.EDITL  .OR.  VS(K,L).LT.EDITL)  GO TO 62
   61 CONTINUE
      GO TO 64
C
C     COMPUTE DISTANCES BETWEEN SURFACE STATION AND UPPER STATIONS
C
   62 J = NSURF + I
c      RSX = (UTMXST(J)  - UTMXST(L))/DXK
c      RSY = (UTMYST(J)  - UTMYST(L))/DYK
      RSX = (UTMXST(J)  - UTMXST(L))/DX      ! in m
      RSY = (UTMYST(J)  - UTMYST(L))/DY
      RS(I) = RSX**2 + RSY**2
      RS(I) = SQRT(RS(I))
   64 CONTINUE
      CALL FMINF(RS,NUPPER,FMIN,NMIN)
      IF(FMIN.LT.RMIN2)  GO TO 70
   63 IF(IABS(IEXTRP).GE.3)  GO TO 68
C
C     EXTRAPOLATE WITH POWER LAW PROFILE
C
      IG = IST(L)
      JG = JST(L)
      DO 67 K=2,NZ
        KM1=K-1
        IF(US(KM1,L).GT.EDITL   .OR.   VS(KM1,L).GT.EDITL)  GO TO 70
        PEXP = 0.143
        IF (HTOPO(IG,JG) .EQ. 0.) PEXP = 0.286
        IF (K.GT.2)  GO TO 65
        PADJ = ( CELLZC(K)/ZSWIND )**PEXP
        GO TO 66
   65   PADJ = ( CELLZC(K)/CELLZC(KM1) )**PEXP
   66   US(K,L) = US(KM1,L)*PADJ
        VS(K,L) = VS(KM1,L)*PADJ
   67 CONTINUE
      GO TO 70
C
C     EXTRAPOLATE WITH USER'S EXTRAPOLATION MULTIPLIERS
C
   68 DO 69 K=2,NZ
        US(K,L) = US(1,L)*FEXTRP(K)
        VS(K,L) = VS(1,L)*FEXTRP(K)
   69 CONTINUE
   70 CONTINUE
   91 CONTINUE
C
C     INITIALIZE GLOBAL ARRAYS                                     .
C
      N = NXMAX*NYMAX*NZMAX
      CALL XMIT(-N,ZERO,U)
      CALL XMIT(-N,ZERO,V)
      CALL XMIT(-N,ZERO,DIV)
      N = NXMAX*NYMAX*NZPMAX
      CALL XMIT(-N,ZERO,W)
C
C     INTERPOLATE WIND FIELD IN EACH VERTICAL LAYER DEPENDING ON THE 
C     DISTANCE TO THE STATIONS             .
C
      IF (I3DCTW .EQ. 1) THEN
        CALL INTER2(US,NWIND,UTMXST,UTMYST,RMIN,RMAX1,RMAX2,RMAX3,
     1  R1,R2,NINTRP,RS,IS,WORK,U,NUMBAR,UG,HTOPO,LNDWTR)
        CALL INTER2(VS,NWIND,UTMXST,UTMYST,RMIN,RMAX1,RMAX2,RMAX3,
     1  R1,R2,NINTRP,RS,IS,WORK,V,NUMBAR,VG,HTOPO,LNDWTR)
      ELSE
        CALL INTERP(US,NWIND,UTMXST,UTMYST,RMIN,RMAX1,NINTRP,RS,IS,
     1              WORK,U,NUMBAR)
        CALL INTERP(VS,NWIND,UTMXST,UTMYST,RMIN,RMAX1,NINTRP,RS,IS,
     1              WORK,V,NUMBAR)
      ENDIF
C
C     OUTPUT INTERPOLATED WIND FIELD                            .
C
c      IF(IPR0.LE.0) GO TO 250
c      CALL WNDPR2(U,V,NZPRNT)
c  250 CONTINUE

C
C     ADJUST SURFACE LAYER WIND FOR TERRAIN EFFECTS
C
      IF(HTFAC.LE.0.) GO TO 285
      IF (I3DCTW .NE. 1) CALL ADJUST(U,V,W,HTOPO,CELLZB(NZP1),
     1                               UB,VB)
C
C     SET BOUNDARY CONDITIONS
C
      DO 1810 K = 1,NZ
        CALL WINDBC(U,V,UB,VB,K)
 1810 CONTINUE
C
C     OUTPUT ADJUSTED WIND FIELD                            .
C
c      IF(IPR1.LE.0) GO TO 285
c      CALL WNDPR2(U,V,NZPRNT)
  285 CONTINUE
C
C     SMOOTH HORIZONTAL FIELDS
C
      CALL SMOOTH(U,V,UB,VB,NSMTH)
C
C     REINITIALIZE VERTICAL VELOCITY
C
      N=NXMAX*NYMAX*NZPMAX
      CALL XMIT(-N,ZERO,W)
C
C     COMPUTE DIVERGENCE FIELDS AND VERTICAL VELOCITIES
C
      DIVMAX=-1.0E+09
      DO 330 K=1,NZ
        CALL DIVCEL(U,V,W,DIV,UB,VB,DIVMAX,K)
  330 CONTINUE
C
      DO 345 K=1,NZ
      DO 340 J=1,NY
      DO 340 I=1,NX
        W(I,J,K+1)=-DZ(K)*DIV(I,J,K)+W(I,J,K)
  340 CONTINUE
C
C     SET BOUNDARY CONDITIONS
C
      CALL WINDBC(U,V,UB,VB,K)
  345 CONTINUE
C
C     ADJUST VERTICAL VELOCITY AS SUGGESTED BY OBRIEN (1970)
C     (JAM, V.9, NO.2), WITH ERROR IN W PROPORTIONAL TO HEIGHT
C
      IF (IOBR .EQ. 1) THEN
      DO 350 K=1,NZ
      DO 350 J=1,NY
      DO 350 I=1,NX
        W(I,J,K+1)=W(I,J,K+1)-CELLZB(K+1)/CELLZB(NZP1)*W(I,J,NZP1)
  350 CONTINUE
      ENDIF
C
C     OUTPUT INITIAL WIND AND DIVERGENCE FIELDS
C
c      IF (IPR2 .LE. 0) GO TO 360
c      CALL WINDPR(U,V,W,NZPRNT)
c      WRITE(IWR,2401) DIVMAX
c      CALL DIVPR(DIV,NZPRNT)
c  360 CONTINUE

C
C     ITERATE TO MINIMIZE DIVERGENCE
C
      IF (IOBR .EQ. 1) CALL MINIM(U,V,W,UB,VB,DIV,NITER,DIVLIM)


C
C     OUTPUT FINAL WIND FIELD
C
      CALL outres(TIME,TIME2,U,V,NXMAX,NYMAX,NZMAX)
C
C     INCREMENT TIME
C
  850 TIME = TIME2 
      if(TIME.GE.TEND) go_on = .false.
  900 end do    ! end do while
!
      return
	end
