      SUBROUTINE SLOPE(USLOPE,VSLOPE,HTOPO,BETA2,TINF,
     1                 TIME,HMAX,GAMMA)
C
C     DIAGNO  VERSION 1.1  LEVEL 900221  SLOPE    S.DOUGLAS, SAI
C
      include 'diagno.inc'
C
      COMMON /D5/ NX,NY,NZ,DX,DY,DZ(NZMAX)
      COMMON /D6/ IRD,IWR,IFILE
      COMMON /S1/ USFC(NXMAX,NYMAX),VSFC(NXMAX,NYMAX)
      DIMENSION USLOPE(NXMAX,NYMAX,1), VSLOPE(NXMAX,NYMAX,1),
     1          HTOPO(NXMAX,1), BETA2(1), HMAX(NXMAX,1),
     2          GAMMA(1)
C
C     THIS SUBROUTINE IS ADAPTED FROM THE 3-D COMPLEX TERRAIN WIND
C     MODEL (YOCKE,1979).  THE U AND V WIND COMPONENTS DUE TO
C     SLOPE FLOW ARE CALCULATED.
C
C     INPUT:  HTOPO (R ARRAY) - GRIDDED TERRAIN HEIGHTS
C             BETA2 (R ARRAY) - FACTORS WHICH DETERMINE THE SIGN
C                               OF THE SLOPE FLOW
C             TINF (R)        - DOMAIN REPRESENTATIVE TEMPERATURE
C             TIME (R)        - SIMULATION TIME
C             HMAX (R ARRAY)  - MAXIMUM TERRAIN HEIGHTS WITHIN A
C                               GIVEN RADIUS
C             GAMMA (R ARRAY) - TEMPERATURE LAPSE RATES (HOURLY )
C
C     OUTPUT:  USLOPE (R ARRAY) - U-COMPONENTS OF THE SLOPE FLOW
C              VSLOPE (R ARRAY) - V-COMPONENTS OF THE SLOPE FLOW
C
      DATA ZERO/0./
C
C     DEFINE CONVERSION FACTORS
C
      RPD=0.017453
      DPR=57.296
C
C     INITIALIZE ARRAYS
C
      N=NXMAX*NYMAX
      CALL XMIT(-N,ZERO,USLOPE)
      CALL XMIT(-N,ZERO,VSLOPE)
C
      DO 100 J=1,NY
      DO 100 I=1,NX
      DELU=0.
      DELV=0.
C
C     CALCULATE THE TOPOGRAPHIC GRADIENTS
C
      DXI=0.5/DX
      DYI=0.5/DY
      HTOIM1=HTOPO(I,J)
      HTOJM1=HTOPO(I,J)
      IF (I .GT. 1) HTOIM1=HTOPO(I-1,J)
      IF (J .GT. 1) HTOJM1=HTOPO(I,J-1)
      HTOIP1=HTOPO(I,J)
      HTOJP1=HTOPO(I,J)
      IF (I .LT. NX) HTOIP1=HTOPO(I+1,J)
      IF (J .LT. NY) HTOJP1=HTOPO(I,J+1)
      DELHI=(HTOIP1-HTOIM1)*DXI
      DELHJ=(HTOJP1-HTOJM1)*DYI
C
C     CALCULATE THE SLOPE FLOW
C
      AALPHA=ATAN((DELHI**2+DELHJ**2)**0.5)
      IF (ABS(AALPHA) .LT. 0.009) GO TO 100
      SINALF=SIN(ABS(AALPHA))
      SINALT=(DELHI**2+DELHJ**2)**0.5
      IF (SINALT .LT. 0.) SINALT=0.
      IT = 1
      TMAX=HMAX(I,J)*GAMMA(IT)
      UVALLY=BETA2(IT)*SQRT(1000.*ABS(TMAX)*SINALT/TINF)
C
C     SET UP DRAINAGE VECTOR DIRECTIONS
C
      IF (DELHI .EQ. 0.) THEN
        IF (DELHJ .EQ. 0.) GO TO 100
        IF (DELHJ .LT. 0.) THET=270.
        IF (DELHJ .GT. 0.) THET=90.
      ELSE
        THETP=ATAN(DELHJ/DELHI)*DPR
      ENDIF
      IF (DELHI .LT. 0.) THET=THETP+180.
      IF (DELHI .GT. 0.) THEN
        IF (DELHJ .GT. 0.) THEN
          THET=THETP
        ELSE
          THET=THETP+360.
        ENDIF
      ENDIF
      IF (THET .GE. 0. .AND. THET .LE. 90.) THETD=90.-THET
      IF (THET .GT. 90. .AND. THET .LE. 360.) THETD=450.-THET
C
C     RESOLVE SLOPE FLOW INTO COMPONENTS
C
      ANG=(270.-THETD)*RPD
      DELU=-COS(ANG)*UVALLY
      DELV=-SIN(ANG)*UVALLY
C
C     SET UP USLOPE AND VSLOPE ARRAYS
C
      USLOPE(I,J,1)=DELU
      VSLOPE(I,J,1)=DELV
C     USLOPE(I,J,2)=0.5*DELU
C     VSLOPE(I,J,2)=0.5*DELV
C     USLOPE(I,J,3)=-0.5*DELU
C     VSLOPE(I,J,3)=-0.5*DELV
  100 CONTINUE
C
      RETURN
      END