      SUBROUTINE TOPOF2(U,V,W,HTOPO,CELLZB,
     1                  GAMMA,TINF,ALPHA,TIME)
C
C     DIAGNO  VERSION 1.1  LEVEL 900221  TOPOF2    S.DOUGLAS, SAI
C
      include 'diagno.inc'
C
      COMMON /D5/ NX,NY,NZ,DX,DY,DZ(NZMAX)
      COMMON /D6/ IRD,IWR,IFILE
      COMMON /T1/ W1(NXMAX,NYMAX,NZPMAX)
      DIMENSION U(NXMAX,NYMAX,1), V(NXMAX,NYMAX,1),
     1          W(NXMAX,NYMAX,1), HTOPO(NXMAX,1), CELLZB(NZPMAX),
     2          GAMMA(1), WSFC(NXMAX,NYMAX)
C
C     THIS SUBROUTINE IS ADAPTED FROM THE 3-D COMPLEX TERRAIN
C     WIND MODEL (YOCKE,1979).  THE VERTICAL VELOCITY DUE TO
C     TOPOGRAPHIC EFFECTS IS COMPUTED AND TRANSFORMED TO TERRAIN
C     FOLLOWING COORDINATES
C
C
C     INPUT:  U (R ARRAY)      - GRIDDED X-DIRECTION WIND COMPONENTS
C             V (R ARRAY)      - GRIDDED Y-DIRECTION WIND COMPONENTS
C             HTOPO (R ARRAY)  - GRIDDED TERRAIN HEIGHTS
C             CELLZB (R ARRAY) - VERTICAL LEVEL HEIGHTS
C             GAMMA (R ARRAY)  - TEMPERATURE LAPSE RATES (HOURLY)
C             TINF (R)         - DOMAIN REPRESENTATIVE TEMPERATURE
C             ALPHA (R)        - EMPIRICAL COEFFICIENT FOR
C                                TOPOGRAPHIC EFFECTS
C             TIME (R)         - SIMULATION TIME
C
C     OUTPUT:  W (R ARRAY) - TERRAIN INDUCED VERTICAL VELOCITY
C
C     CALCULATE THE STABILITY PARAMETER, S
C
      TAU=-0.01
c      IT = IFIX(TIME/100.) + 1
      IT = 1 
      GAMMA2=GAMMA(IT)-TAU
      IF (GAMMA2 .LT. 0.) S=-1.0
      IF (GAMMA2 .GE. 0.) S=(9.8*GAMMA2/TINF)**0.5
C
C     CALCULATE THE VERTICAL VELOCITY
C
      DO 100 J=1,NY
      DO 100 I=1,NX
C
C     CALCULATE THE WAVENUMBER OF THE ATMOSPHERE
C
      HINV = 500.
      IF (S .LE. 0.) THEN
        BK=2./HINV
      ELSE
        BK = S/((U(I,J,NZ)**2+V(I,J,NZ)**2)**0.5)
      ENDIF
C
C     CALCULATE THE TOPOGRAPHIC GRADIENTS
C
      DXI=0.5/DX
      DYI=0.5/DY
      HTOIM1=HTOPO(I,J)
      HTOJM1=HTOPO(I,J)
      IF (I .GT. 1) HTOIM1=HTOPO(I-1,J)
      IF (J .GT. 1) HTOJM1=HTOPO(I,J-1)
      HTOIP1=HTOPO(I,J)
      HTOJP1=HTOPO(I,J)
      IF (I .LT. NX) HTOIP1=HTOPO(I+1,J)
      IF (J .LT. NY) HTOJP1=HTOPO(I,J+1)
      DELHI=(HTOIP1-HTOIM1)*DXI
      DELHJ=(HTOJP1-HTOJM1)*DYI
C
C     CALCULATE THE VERTICAL VELOCITY DUE TO TOPOGRAPHIC EFFECTS
C
      WTOPO=U(I,J,1)*DELHI+V(I,J,1)*DELHJ
C
C     SET THE LOWER BOUNDARY CONDITION
C
      W1(I,J,1)=WTOPO
      W(I,J,1)=WTOPO
C
C     COMPUTE THE EXPONENTIAL DECAY OF VERTICAL VELOCITY
C
      DO 120 K=1,NZ
      KP1=K+1
      BKZ=BK*CELLZB(KP1)
      IF (BKZ .GT. 100.) BKZ=100.
      W1(I,J,KP1)=WTOPO*EXP(-BKZ)
C
C     CALCULATE DWDZ
C
      DZI=1.0/DZ(K)
      DWDZ1=(W1(I,J,KP1)-W1(I,J,K))*DZI
      DWDZ=ALPHA*DWDZ1
C
C     CALCULATE W FROM DWDZ
C
      W(I,J,KP1)=DWDZ/DZI+W(I,J,K)
  120 CONTINUE
C
C
C     TRANSFORM TO TERRAIN FOLLOWING COORDINATES
C
      W(I,J,1)=W(I,J,1)-U(I,J,1)*DELHI-V(I,J,1)*DELHJ
      DO 130 K=1,NZ
      KP1=K+1
      W(I,J,KP1)=W(I,J,KP1)-U(I,J,K)*DELHI-V(I,J,K)*DELHJ
  130 CONTINUE
  100 CONTINUE
      RETURN
      END