!***************************************************************
!*
!*		Module for input/output
!*
!***************************************************************
 MODULE io_parameters
    IMPLICIT NONE
    SAVE
!
!***  File logical units
!
     integer(kind=4), parameter  :: nlog = 1      ! log          file
     integer(kind=4), parameter  :: ninp = 10     ! input        file
!
!***  File names
!
     character(len=250) :: flog,finp,ftop,frou,fmet,fdia,frst,fsrc,fpts,fbox,dump  ! input file names
!
!***  Postprocess variables
!
     logical      :: out_dom,out_src,out_h  ,out_u  ,out_v,out_rho,out_dose, &
                     out_c  ,out_z  ,out_pts,out_box
     real(kind=8) :: tevery,c_bg                                                
!
!***  Concentration points
!
     integer(kind=4) :: nhts,ncri
!
     real(kind=8),allocatable :: z_hts(:)      ! z(nhts) z-coordinates for concentration heights
     real(kind=8),allocatable :: c_cri(:)      ! c(ncri) Critical concentration values
!
!***  Tracking points
!
     integer(kind=4) :: npts
     integer(kind=4), allocatable :: i_pts(:)     ! mesh index for track points
     integer(kind=4), allocatable :: j_pts(:)     ! mesh index for track points
!
     real(kind=8), allocatable :: x_pts(:)     ! x(npts)  x-coordinates for track points
     real(kind=8), allocatable :: y_pts(:)     ! y(npts)  y-coordinates for track points
     real(kind=8), allocatable :: z_pts(:)     ! z(npts)  z-coordinates for track points
     real(kind=8), allocatable :: s_pts(:,:)   ! interpolation shape function for tracking points
!
!***  Tracking boxes
!
     integer(kind=4) :: nbox
     integer(kind=4), allocatable :: i1_box(:)     ! mesh index for box
     integer(kind=4), allocatable :: j1_box(:)     ! mesh index for box
     integer(kind=4), allocatable :: i2_box(:)     ! mesh index for box
     integer(kind=4), allocatable :: j2_box(:)     ! mesh index for box
!
     real(kind=8), allocatable :: x_box(:)     ! x (nbox)  x-coordinates for track boxes
     real(kind=8), allocatable :: y_box(:)     ! y (nbox)  y-coordinates for track boxes 
     real(kind=8), allocatable :: z_box(:)     ! z (nbox)  z-coordinates for track boxes
!
CONTAINS
!
!
!
      subroutine get_input_npar(fname,block,line,npar,istat,message)
!**************************************************************************
!*
!*    Gets the number of parameters associated to a line
!*
!*    INPUT:
!*    character*(*)   fname    Name of the file
!*    character*(*)   block    Block to search
!*    character*(*)   line     Line block to search
!*
!*    OUTPUT:
!*    integer*4       npar     Number of parameters                     
!*    integer*4       istat    0 if OK
!*    character*(*)   message  Output message with the error description       
!*
!*
!**************************************************************************
      implicit none
!
      character(len=100) :: message
      character*(*)      :: fname,block,line
      integer(kind=4)    :: npar,istat
!
      logical            :: linefound,blockfound
      integer(kind=4)    :: nword
      character(len=256) :: card
      character(len=256) :: words(128)
      real     (kind=8 ) :: param(128)
!
!***  Initializations
!
      istat = -1
!
!***  Opens the file
!
      open(90,FILE=TRIM(fname),STATUS='old',ERR=101)
!
!***  Search the line
!
      blockfound = .false.
      linefound  = .false.
      do while(.not.linefound)
        do while(.not.blockfound) 
             read(90,'(a256)',END=102) card
             call decode(card,words,param,nword,npar)
             if(words(1)(1:LEN_TRIM(block)).eq.block(1:LEN_TRIM(block))) blockfound=.true.
         end do
         read(90,'(a256)',END=103) card
         call decode(card,words,param,nword,npar)
         if(words(1)(1:LEN_TRIM(line)).eq.line(1:LEN_TRIM(line))) linefound = .true.
      end do
!
!***  Successful end
!	 
      close(90)
      istat = 0
      message = ' '
      return
!
!***  List of errors
!
  101 message = 'Error opening the input file '//TRIM(fname)
      close(90)
      return
  102 message = 'Block '//TRIM(block)//' not found in the input file' 
      close(90)
      return
  103 message = 'Line '//TRIM(line)//' not found in the input file' 
      close(90)
      return
      end subroutine get_input_npar

  subroutine get_input_int4(fname,block,line,value,nval,istat,message)
!**************************************************************************
!*
!*    Gets nval integer*4 inputs from the file fname
!*
!*    INPUT:
!*    character*(*)   fname    Name of the file
!*    character*(*)   block    Block to search
!*    character*(*)   line     Line block to search
!*    integer*4       nval     Number of integers to read                     
!*
!*    OUTPUT:
!*    integer*4       istat    0 if OK
!*    integer*4       value    Values of the nval integers read
!*    character*100   message  Output message with the error description       
!*
!**************************************************************************
    implicit none
!
    character(len=100) :: message
    character*(*)      :: fname,block,line
    integer(kind=4)    :: nval,istat
    integer(kind=4)    :: value
!
    logical            :: linefound,blockfound
    integer(kind=4)    :: nword,npar
    character(len=256) :: card
    character(len=256) :: words(128)
    real     (kind=8 ) :: param(128)
!
!***  Initializations
!
    istat = -1
!
!***  Opens the file
!
    open(90,FILE=TRIM(fname),STATUS='old',ERR=101)
!
!***  Search the line
!
    blockfound = .false.
    linefound  = .false.
    do while(.not.linefound)
     do while(.not.blockfound) 
          read(90,'(a256)',END=102) card
          call decode(card,words,param,nword,npar)
          if(words(1)(1:LEN_TRIM(block)).eq.block(1:LEN_TRIM(block))) blockfound=.true.
       end do
       read(90,'(a256)',END=103) card
       call decode(card,words,param,nword,npar)
       if(words(1)(1:LEN_TRIM(line)).eq.line(1:LEN_TRIM(line))) linefound = .true.
    end do
!                 
    if(npar.lt.nval) goto 104
!
!      do ival = 1,nval
!	     value(ival) = INT(param(ival))
!      end do
    value = INT(param(1))
!
!***  Successful end
!	 
    close(90)
    istat = 0
    message = ' '
    return 
!
!***  List of errors
!
101 message = 'get_input_int4: error opening the input file '//TRIM(fname)
    return
102 message = 'get_input_int4: block '//TRIM(block)//' not found in the input file' 
    return
103 message = 'get_input_int4: line '//TRIM(line)//' not found in the input file' 
    return
104 message = 'get_input_int4: too few parameters in line '//TRIM(line) 
    return
  end subroutine get_input_int4
!
!
!
  subroutine get_input_rea8(fname,block,line,value,nval,istat,message)
!**************************************************************************
!*
!*    Gets nval real*8 inputs from the file fname
!*
!*    INPUT:
!*    character*(*)   fname    Name of the file
!*    character*(*)   block    Block to search
!*    character*(*)   line     Line block to search
!*    integer*4       nval     Number of integers to read                     
!*
!*    OUTPUT:
!*    integer*4       istat    0 if OK
!*    real*8          value    Values of the nval integers read
!*    character*100   message  Output message with the error description       
!*
!**************************************************************************
    implicit none
!
    character(len=100) :: message
    character*(*)      :: fname,block,line
    integer(kind=4)    :: nval,istat
    real   (kind=8)    :: value
!
    logical            :: linefound,blockfound
    integer  (kind=4)  :: nword,npar
    character(len=256) :: card
    character(len=256) :: words(128)
    real     (kind=8)  :: param(128)
!
!***  Initializations
!
    istat = -1
!
!***  Opens the file
!
    open(90,FILE=TRIM(fname),STATUS='old',ERR=101)
!
!***  Search the line
!
    blockfound = .false.
    linefound  = .false.
    do while(.not.linefound)
       do while(.not.blockfound) 
          read(90,'(a256)',END=102) card
          call decode(card,words,param,nword,npar)
          if(words(1)(1:LEN_TRIM(block)).eq.block(1:LEN_TRIM(block))) blockfound=.true.
       end do
       read(90,'(a256)',END=103) card
       call decode(card,words,param,nword,npar)
       if(words(1)(1:LEN_TRIM(line)).eq.line(1:LEN_TRIM(line))) linefound = .true.
    end do
!   
    if(nval.eq.1) value = param(1)

!
!***  Successful end
!	 
    close(90)
    istat = 0
    message = 'OK'
    return
!
!***  List of errors
!
101 message = 'Error opening the input file '//TRIM(fname)
    return
102 message = 'Block '//TRIM(block)//' not found in the input file' 
    return
103 message = 'Line '//TRIM(line)//' not found in the input file' 
    return
104 message = 'Number of parameters in line '//TRIM(line)//' is less than required' 
    return
  end subroutine get_input_rea8
!
!
!
  subroutine get_input_vec8(fname,block,line,value,nval,istat,message)
!**************************************************************************
!*
!*    Gets nval real*8 inputs from the file fname
!*
!*    INPUT:
!*    character*(*)   fname    Name of the file
!*    character*(*)   block    Block to search
!*    character*(*)   line     Line block to search
!*    integer*4       nval     Number of integers to read                     
!*
!*    OUTPUT:
!*    integer*4       istat    0 if OK
!*    real*8          value    Values of the nval integers read
!*    character*100   message  Output message with the error description       
!*
!**************************************************************************
    implicit none
!
    character(len=100) :: message
    character*(*)      :: fname,block,line
    integer(kind=4)    :: nval,istat
    real   (kind=8)    :: value(nval)
!
    logical            :: linefound,blockfound
    integer  (kind=4)  :: nword,npar,ival
    character(len=256) :: card
    character(len=256) :: words(128)
    real     (kind=8)  :: param(128)
!
!***  Initializations
!
    istat = -1
!
!***  Opens the file
!
    open(90,FILE=TRIM(fname),STATUS='old',ERR=101)
!
!***  Search the line
!
    blockfound = .false.
    linefound  = .false.
    do while(.not.linefound)
       do while(.not.blockfound) 
          read(90,'(a256)',END=102) card
          call decode(card,words,param,nword,npar)
          if(words(1)(1:LEN_TRIM(block)).eq.block(1:LEN_TRIM(block))) blockfound=.true.
       end do
       read(90,'(a256)',END=103) card
       call decode(card,words,param,nword,npar)
       if(words(1)(1:LEN_TRIM(line)).eq.line(1:LEN_TRIM(line))) linefound = .true.
    end do
!                 
!    if(npar.lt.nval) goto 104
!
     if(nval.gt.1) then
       do ival = 1,nval
          value(ival) = param(ival)
       end do
     else
        value(1) = param(1)
    end if
!
!***  Successful end
!	 
    close(90)
    istat = 0
    message = 'OK'
    return
!
!***  List of errors
!
101 message = 'Error opening the input file '//TRIM(fname)
    return
102 message = 'Block '//TRIM(block)//' not found in the input file' 
    return
103 message = 'Line '//TRIM(line)//' not found in the input file' 
    return
104 message = 'Number of parameters in line '//TRIM(line)//' is less than required' 
    return
  end subroutine get_input_vec8
!
!
!
  subroutine get_input_cha(fname,block,line,value,nval,istat,message)
!**************************************************************************
!*
!*    Gets nval character inputs from the file fname
!*
!*    NOTE: words are converted to UPPER case
!*
!*    INPUT:
!*    character*(*)   fname    Name of the file
!*    character*(*)   block    Block to search
!*    character*(*)   line     Line block to search
!*    integer*4       nval     Number of integers to read                     
!*
!*    OUTPUT:
!*    integer*4       istat    0 if OK
!*    character*(*)   value    Values of the nval integers read
!*    character*100   message  Output message with the error description       
!*
!**************************************************************************
    implicit none
!
    integer(kind=4)    :: nval,istat
    character(len=100) :: message
    character*(*)      :: fname,block,line
    character*(*)      :: value
!
    logical            :: linefound,blockfound
    integer(kind=4)    :: nword,npar,j
    character(len=256) :: card
    character(len=256) :: words(128)
    real(kind=8)       :: param(128)
!
!***  Initializations
!
    istat = -1
!
!***  Opens the file
!
    open(90,FILE=TRIM(fname),STATUS='old',ERR=101)
!
!***  Search the line
!
    blockfound = .false.
    linefound  = .false.
    do while(.not.linefound)
       do while(.not.blockfound) 
          read(90,'(a256)',END=102) card
          call decode(card,words,param,nword,npar)
          if(words(1)(1:LEN_TRIM(block)).eq.block(1:LEN_TRIM(block))) blockfound=.true.
       end do
       read(90,'(a256)',END=103) card
       call decode(card,words,param,nword,npar)
       if(words(1)(1:LEN_TRIM(line)).eq.line(1:LEN_TRIM(line))) linefound = .true.
    end do
!  
      if((nword-1).lt.nval) goto 104
!
      value(1:LEN_TRIM(words(2))) = words(2)(1:LEN_TRIM(words(2)))
!
!***     Fill the rest with ' '
!
      do j=LEN_TRIM(words(2))+1,LEN(value)
         value(j:j)=' '
      end do
!
!***  Successful end
!	 
      close(90)
      istat = 0
      message = 'OK'
      return
!
!***  List of errors
!
101   message = 'get_input_cha: error opening the input file '//TRIM(fname)
      return
102   message = 'get_input_cha: block '//TRIM(block)//' not found in the input file' 
      return
103   message = 'get_input_cha: line '//TRIM(line)//' not found in the input file' 
      return
104   message = 'get_input_cha: too few parameters in line '//TRIM(line) 
      return
    end subroutine get_input_cha
!
    subroutine decode(card,words,param,nword,npar)
!********************************************************************
!*
!*  This routine decodes a string card(256) into words and parameters 
!*
!********************************************************************
    implicit none
    integer(kind=4)    :: nword,npar
    character(len=256) :: card
    character(len=256) :: words(128)
    character(len=256) :: s(128)
    real(kind=8)       :: param(128)
!
    integer(kind=4):: ipos,first,last,nstr,lflag,i
    real(kind=8)   :: digit
!
!***  Initializations
!
    nword = 0
    npar  = 0     
    ipos  = 0
!
    do while(1.ne.0)
!                                    ! First position
         ipos = ipos + 1
         if(ipos.gt.256) return 
  10     if(card(ipos:ipos).eq.' '.or.card(ipos:ipos).eq.'=') then
         ipos = ipos + 1 
         if(ipos.gt.256) return 
           go to 10
         end if
         first = ipos
!
         ipos = ipos + 1
         if(ipos.gt.256) return 
  20     if(card(ipos:ipos).ne.' '.and.card(ipos:ipos).ne.'=') then
         ipos = ipos + 1 
         if(ipos.gt.256) return 
           go to 20
         end if
         last = ipos-1
!
         nstr = last-first+1
         s(1:nstr) = card(first:last)
         call decod1(s,nstr,lflag,digit)
         if(lflag.eq.0) then
            npar = npar + 1
            param(npar)= digit
        else if(lflag.eq.1) then
            nword = nword + 1
            do i=1,256
               words(nword)(i:i) = ' '
        end do
         words(nword)(1:nstr) = card(first:last)
        end if
!
    end do
    end subroutine decode
!
    subroutine decod1(string,nstr,lflag,digit)
!*******************************************************************
!*
!*    This subroutine decodes a single string(1:nstr) 
!*    
!*    If string(1:nstr) is a string returns lflag = 1
!*    If string(1:nstr) is a number returns lflag = 0 and the digit
!*
!*******************************************************************
      implicit none
      integer(kind=4) :: nstr,lflag
      integer(kind=4) :: istr,decim
      real(kind=8)    :: digit
      character(len=1):: string(nstr)
!	  
      lflag = 0                                             ! Number by default
      istr  = 1
!      
      do while(istr.le.nstr)
        decim = ichar(string(istr))                         ! decimal value
        if(decim.lt.48.or.decim.gt.57) then                 ! It is not a num.
           if(decim.ne.43 .and.decim.ne.45 .and. &          ! discard + -
              decim.ne.68 .and.decim.ne.69 .and. &          ! discard D E
              decim.ne.100.and.decim.ne.101.and. &          ! discard d e
              decim.ne.46) then                             ! discard .
              lflag = 1
              istr  = nstr
        end if
        end if
        istr = istr+1
      end do
!
      if(lflag.eq.0) then                                   ! It's a number
        digit = stof(string,nstr)
      end if
!
      return
      end subroutine decod1
!
      real(kind=8) function stof(string,nstr)
!**************************************************************
!*
!*    This routine converts a real/integer number stored in a
!*    string(1:nstr) into a real*8 digit format
!*
!**************************************************************
      implicit none
      integer(kind=4) ::  nstr
      character(len=1)::  string(*)
!
      integer(kind=4):: i,ipos,nsign,esign,nvalu
      integer(kind=4):: expo,valu(256)  
      logical next
!
      stof = 0.0
!
!***  Sing decoding
!
      ipos = 1
      if(ichar(string(ipos)).eq.43) then         !  + sign
        nsign = 1
        ipos  = ipos + 1
      else if(ichar(string(ipos)).eq.45) then    !  - sign
        nsign = -1
       ipos  = ipos + 1
      else                                       !  no sing (+)
        nsign = 1
        ipos  = ipos 
      end if
!
!***  Base decoding
!
      nvalu = 0
      next  = .true.
      do while(next)
        if((ichar(string(ipos)).eq.68 ).or.  &       ! D
           (ichar(string(ipos)).eq.69 ).or.  &      ! E
           (ichar(string(ipos)).eq.100).or.  &      ! d
           (ichar(string(ipos)).eq.101).or.  &      ! e
           (ichar(string(ipos)).eq.46 )) then       ! .
           next = .false.
        else
         nvalu = nvalu + 1
         valu(nvalu) = stof1(string(ipos))
         ipos = ipos + 1 
         if(ipos.eq.(nstr+1)) then
           next = .false.
           ipos = ipos - 1
        end if
        end if
      end do
      do i = 1,nvalu
         stof = stof + valu(i)*1d1**(nvalu-i)
      end do
!
!***  Decimal decoding
!
      if((ichar(string(ipos)).eq.46   ).and. &
                      ipos  .ne.nstr) then
        ipos = ipos + 1
        nvalu = 0
        next  = .true.
        do while(next)
          if((ichar(string(ipos)).eq.68 ).or.   &      ! D
             (ichar(string(ipos)).eq.69 ).or.   &      ! E
             (ichar(string(ipos)).eq.100).or.   &      ! d
             (ichar(string(ipos)).eq.101)) then        ! e
             next = .false.
        else
         nvalu = nvalu + 1
         valu(nvalu) = stof1(string(ipos))
         ipos = ipos + 1 
         if(ipos.eq.(nstr+1)) then
         next = .false. 
         ipos = ipos - 1
        end if
        end if
       end do
       do i = 1,nvalu
          stof = stof + valu(i)*1d1**(-i)
       end do
       end if
!
!***  Exponent
!
      if(((ichar(string(ipos)).eq.68 ).or.   &      ! D
          (ichar(string(ipos)).eq.69 ).or.   &      ! E
          (ichar(string(ipos)).eq.100).or.   &      ! d
          (ichar(string(ipos)).eq.101)).and. &      ! e
                        ipos  .ne.nstr) then
        ipos = ipos + 1
        if(ichar(string(ipos)).eq.43) then         !  + sign
           esign = 1
          ipos  = ipos + 1
        else if(ichar(string(ipos)).eq.45) then    !  - sign
           esign = -1
           ipos  = ipos + 1
        else                                       !  no sing (+)
           esign = 1
           ipos  = ipos 
       end if     
!        
        nvalu = 0
        next  = .true.
        do while(next)
           nvalu = nvalu + 1
           valu(nvalu) = stof1(string(ipos))
           ipos = ipos + 1
           if(ipos.eq.(nstr+1)) then
            next = .false.
              ipos = ipos - 1
           end if
           end do
           expo = 0
       do i = 1,nvalu
          expo = expo + valu(i)*1d1**(nvalu-i)
       end do
       expo = esign*expo
!
        stof = stof*(1d1**expo)
       end if
!
      stof = nsign*stof
      end function stof
!
    integer(kind=4) function stof1(string1)
!**************************************************************
!*
!*    Decodes a character*1 string
!*
!**************************************************************
    implicit none
    character(len=1):: string1
!
    if(string1.eq.'0') then
      stof1 = 0
    else if(string1.eq.'1') then
      stof1 = 1
    else if(string1.eq.'2') then
      stof1 = 2
    else if(string1.eq.'3') then
      stof1 = 3
    else if(string1.eq.'4') then
      stof1 = 4
    else if(string1.eq.'5') then
      stof1 = 5
    else if(string1.eq.'6') then
      stof1 = 6
    else if(string1.eq.'7') then
      stof1 = 7
    else if(string1.eq.'8') then
      stof1 = 8
    else if(string1.eq.'9') then
      stof1 = 9
    end if
    return
    end function stof1

END MODULE io_parameters
