!***************************************************************
!*
!*		Module for master operations
!* 
!***************************************************************
  MODULE master_variables
     IMPLICIT NONE
     SAVE
!
!***  Parameters
!
      logical                :: debug = .false.
      real(kind=8),parameter :: zero = 0.0
      real(kind=8),parameter :: pi = 3.14159265358979
      real(kind=8),parameter :: gi = 9.816
!
!***  Time related variables 
! 
      logical         :: restart
      integer(kind=4) :: ibyr,ibmo,ibdy,ibhr,ibmi
      integer(kind=4) :: icounter,ocounter,hflag,rhoflag
      real   (kind=8) :: tbeg,trun                                                
      real   (kind=8) :: time,dt,meteo_time                                                
!
!***  Other time related variables
!
      character(len=3), dimension(12) :: month = (/'JAN','FEB','MAR','APR','MAY','JUN','JUL','AUG','SEP','OCT','NOV','DEC' /)
      real(kind=8) :: cpu_time_begin,cpu_time_end
!
!***  Grid variables
!
      logical         :: topofile
      integer(kind=4) :: nx,ny
      integer(kind=4) :: nx1,nx2,ny1,ny2
      real   (kind=8) :: dx,dy,x0,y0
!
!***  Properties variables
!
      real   (kind=8) :: rhoa,rhoa_20,rho1,rho1_20,Tmean,xexp
!
!***  Numeric parameters
!
      real   (kind=8) :: beta,courant,train,diffus,shape,zeta,xal2,xal3,xal7,vkarm,brit
!
!***  Meteo parameters
!
      character(len =10) :: wind_model
      real     (kind=8 ) :: zref
      real     (kind=8 ) :: Tz0,Tzref,pres
!
!***  Vectors with dimensions (1:nx,1:ny)
!
      real(kind=8), allocatable :: g     (:,:)     ! g       topography
      real(kind=8), allocatable :: zrough(:,:)     ! zrough  Roughness
      real(kind=8), allocatable :: up    (:,:)     ! up      source up velocity
      real(kind=8), allocatable :: windu (:,:)     ! windu   u-velocity wind
      real(kind=8), allocatable :: windv (:,:)     ! windv   v-velocity wind
      real(kind=8), allocatable :: ustar (:,:)     ! ustar   ustar
      real(kind=8), allocatable :: wstar (:,:)     ! wstar   wstar
      real(kind=8), allocatable :: rmonin(:,:)     ! rmonin  Monin-Obukhov lenght
      real(kind=8), allocatable :: cd    (:,:)     ! cd      drag coefficient
      real(kind=8), allocatable :: dose  (:,:)     ! dose    Toxic gas dose 
      real(kind=8), allocatable :: work8 (:,:)      
      real(kind=8), allocatable :: pplus (:,:)      
      real(kind=8), allocatable :: pminus(:,:)      
      real(kind=8), allocatable :: wmax  (:,:)      
      real(kind=8), allocatable :: wmin  (:,:)      
      real(kind=8), allocatable :: rplus (:,:)      
      real(kind=8), allocatable :: rminus(:,:)      
      real(kind=8), allocatable :: cu    (:,:)      
      real(kind=8), allocatable :: cv    (:,:)      
      real(kind=8), allocatable :: utop  (:,:)      
      real(kind=8), allocatable :: u     (:,:)     ! u       average u-velocity
      real(kind=8), allocatable :: v     (:,:)     ! v       average v-velocity
      real(kind=8), allocatable :: h     (:,:)     ! h       average cloud thickness
      real(kind=8), allocatable :: rho   (:,:)     ! rho     density
      real(kind=8), allocatable :: unew  (:,:)     ! unew  
      real(kind=8), allocatable :: vnew  (:,:)     ! vnew    
      real(kind=8), allocatable :: hnew  (:,:)     ! hnew    
      real(kind=8), allocatable :: rhonew(:,:)     ! rhonew 
      real(kind=8), allocatable :: w     (:,:)     ! w      
      real(kind=8), allocatable :: wtd   (:,:)     ! wtd     
      real(kind=8), allocatable :: flu   (:,:)     ! flu     Low -order u flux  
      real(kind=8), allocatable :: flv   (:,:)     ! flv     Low -order v flux  
      real(kind=8), allocatable :: fhu   (:,:)     ! flu     High-order u flux  
      real(kind=8), allocatable :: fhv   (:,:)     ! flv     High-order v flux  
!
      real(kind=4), allocatable :: work4 (:,:,:)
!
 CONTAINS
!
      subroutine getmem
!**********************************************************************
!*
!*    Allocates memory for arrays related to nx,ny
!*
!**********************************************************************
      implicit none
!
      allocate(g(nx,ny))
      g(1:nx,1:ny) = 0.0
!
      allocate(zrough(nx,ny))
      zrough(1:nx,1:ny) = 0.0
!
      allocate(up(nx,ny))
      up(1:nx,1:ny) = 0.0
!
      allocate(windu(nx,ny))
      windu(1:nx,1:ny) = 0.0
!
      allocate(windv(nx,ny))
      windv(1:nx,1:ny) = 0.0
!
      allocate(ustar(nx,ny))
      ustar(1:nx,1:ny) = 0.0
!
      allocate(wstar(nx,ny))
      wstar(1:nx,1:ny) = 0.0
!
      allocate(rmonin(nx,ny))
      rmonin(1:nx,1:ny) = 0.0
!
      allocate(cd(nx,ny))
      cd(1:nx,1:ny) = 0.0
!
      allocate(dose(nx,ny))
      dose(1:nx,1:ny) = 0.0
!
      allocate(work8(nx,ny))
      work8(1:nx,1:ny) = 0.0
!
      allocate(work4(nx,ny,2))
      work4(1:nx,1:ny,1:2) = 0.0
!
      allocate(u(1:nx,1:ny))
      u(1:nx,1:ny) = 0.0
!
      allocate(v(1:nx,1:ny))
      v(1:nx,1:ny) = 0.0
!
      allocate(h(1:nx,1:ny))
      h(1:nx,1:ny) = 0.0
!
      allocate(rho(1:nx,1:ny))
      rho(1:nx,1:ny) = 0.0
!
      allocate(unew(1:nx,1:ny))
      unew(1:nx,1:ny) = 0.0
!
      allocate(vnew(1:nx,1:ny))
      vnew(1:nx,1:ny) = 0.0
!
      allocate(hnew(1:nx,1:ny))
      hnew(1:nx,1:ny) = 0.0
!
      allocate(rhonew(1:nx,1:ny))
      rhonew(1:nx,1:ny) = 0.0
!
      allocate(w(1:nx,1:ny))
      w(1:nx,1:ny) = 0.0
!
      allocate(wtd(1:nx,1:ny))
      wtd(1:nx,1:ny) = 0.0
!
      allocate(flu(1:nx,1:ny))
      flu(1:nx,1:ny) = 0.0
!
      allocate(flv(1:nx,1:ny))
      flv(1:nx,1:ny) = 0.0
!
      allocate(fhu(1:nx,1:ny))
      fhu(1:nx,1:ny) = 0.0
!
      allocate(fhv(1:nx,1:ny))
      fhv(1:nx,1:ny) = 0.0
!
      allocate(pplus(nx,ny))
      pplus(1:nx,1:ny) = 0.0
!
      allocate(pminus(nx,ny))
      pminus(1:nx,1:ny) = 0.0
!
      allocate(wmax(nx,ny))
      wmax(1:nx,1:ny) = 0.0
!
      allocate(wmin(nx,ny))
      wmin(1:nx,1:ny) = 0.0
!
      allocate(rplus(nx,ny))
      rplus(1:nx,1:ny) = 0.0
!
      allocate(rminus(nx,ny))
      rminus(1:nx,1:ny) = 0.0
!
      allocate(cu(nx,ny))
      cu(1:nx,1:ny) = 0.0
!
      allocate(cv(nx,ny))
      cv(1:nx,1:ny) = 0.0
!
      allocate(utop(nx,ny))
      utop(1:nx,1:ny) = 0.0
!
      end subroutine getmem
!	  
 END MODULE master_variables    
