       program twodee2
!****************************************************************************
!*
!*     TWODEE2: A FORTRAN CODE TO COMPUTE HEAVY GAS DISPERSION    
!*              Modified after TWODEE by Robin Hankin      
!*
!*     AUTHORS: A.Folch (afolch@ov.ingv.it) and A.Costa (costa@ov.ingv.it)  
!*     DATE   : MAY-2007
!*
!*     Program to solve two dimensional shallow water equations     
!*     using Zalesak's flux correction algorithm based on 
!*     Hankin & Britter 1999a,1999b,1999c, Journal of Hazardous Materials
!*
!*****************************************************************************
       use master_variables
       use io_parameters
       implicit none
!
       logical :: go_on
!
!***   Read input data
!
       call setup
!
!***   Begin time loop
!
       meteo_time = time
       go_on      = .true. 
!
       do while(go_on) 
!
!***      If necessary, reads meteo data
!         
          if(meteo_time.le.time) then
             if(TRIM(wind_model).eq.'UNIFORM'.or.TRIM(wind_model).eq.'uniform') then ! uniform wind
                call readuni
             else if(TRIM(wind_model).eq.'DIAGNO'.or.TRIM(wind_model).eq.'diagno') then    ! diagno
                call readdwm  
             else
                call runend(-1,'No wind model provided')
             end if
!
             if(debug) call dbg_fixval
             call wriwind
          end if
!
!***      Computations
!
          call tstepsize
          call timestep
          call boundary
          call dose_calc
!
!***      Postprocess and update 
!
          if((time+dt).ge.(ocounter*tevery)) then
          call outres
          ocounter = ocounter + 1
          end if
! 
          if(mod(time,60.0  ).le.dt) call writim
!
          time    = time + dt  
          icounter = icounter + 1
          if(time.ge.trun) go_on = .false.       
!
         end do
!
!***   End run
!
         call writim
         call outres
         call runend(1,'OK')

       end program twodee2
       

