      subroutine bothfluxschemes_w
!****************************************************************************
!*
!*    This routine fills the flu,flv,fhu,fhv arrays. The fh means high order
!*    and the u- or v- means the u- or v- flux. The first attempt at the high
!*    order scheme will be a fourth order ZIP effort. The low order scheme 
!*    will be an upwind differencing scheme
!*    The convention used is that fhu(i,j) is the u flux corresponding to 
!*    the boundary (i+1/2,j).
!*
!****************************************************************************
      use master_variables
      implicit none
!
      integer(kind=4) :: i,j
      real   (kind=8) :: ubar,vbar,wdc,f1,f2
!     real   (kind=8) :: speedi,speedj,h_avei,h_avej,xexch
!
      f1 = 2.0/ 3.0
      f2 = 1.0/12.0
! 
      do j = ny1,ny2
         do i = nx1,nx2
!
!***        Higher order
!
            fhu(i,j)=(f1*(w(i+1,j)*u(i  ,j)+w(i  ,j)*u(i+1,j))       &
                     -f2*(w(i+2,j)*u(i  ,j)+w(i  ,j)*u(i+2,j)+       &
                          w(i+1,j)*u(i-1,j)+w(i-1,j)*u(i+1,j)))*dt
!      
            fhv(i,j)=(f1*(w(i,j+1)*v(i,j  )+w(i,j  )*v(i,j+1))       &
                     -f2*(w(i,j+2)*v(i,j  )+w(i,j  )*v(i,j+2)+       &
                          w(i,j+1)*v(i,j-1)+w(i,j-1)*v(i,j+1)))*dt
!
!           The above from Zalesak, JCP, 40/500/1981.
!           The next part is an addition to flu and fhu that
!           diffuses density from one part to another. This is a sort
!           of Fickian diffusion term, only present in the high order
!           part as it is not guaranteed to be monotonic.
!
!            xexch=0.0 
!            if (rhoflag .EQ. 1)then
!               speedi = 0.7071*sqrt(u(i  ,j)*u(i  ,j)+v(i  ,j)*v(i  ,j)+  &
!                                      u(ip1,j)*u(ip1,j)+v(ip1,j)*v(ip1,j))
!               speedj = 0.7071*sqrt(u(i,j  )*u(i,j  )+v(i,j  )*v(i,j  )+  &
!                                      u(i,jp1)*u(i,jp1)+v(i,jp1)*v(i,jp1))
!               h_avei = 0.7071*sqrt(h(i,j)*h(i,j)+h(ip1,j)*h(ip1,j))
!               h_avej = 0.7071*sqrt(h(i,j)*h(i,j)+h(i,jp1)*h(i,jp1))
!
!               fhu(i,j) = fhu(i,j) + xexch*speedi*h_avei*(rho(i,j)-rho(ip1,j))
!               fhv(i,j) = fhv(i,j) + xexch*speedj*h_avej*(rho(i,j)-rho(i,jp1))
!            endif
!
!***        Lower order
!
            ubar= 0.5*(u(i,j)+u(i+1,j))
            if(ubar .GE. 0.0)then
               wdc=w(i,j)
            else
               wdc=w(i+1,j)
            endif
            flu(i,j) = ubar*wdc*dt - dx*(w(i+1,j)-w(i,j))*diffus
!
            vbar= 0.5*(v(i,j)+v(i,j+1))
            if(vbar .GE. 0.0)then
               wdc=w(i,j)
            else
               wdc=w(i,j+1)
            endif
            flv(i,j) = vbar*wdc*dt - dy*(w(i,j+1)-w(i,j))*diffus
!
!***
!
            if(h(i,j)+g(i,j) .LT. g(i+1,j))then
               if(w(i+1,j) .GE. 0.0)then
                  flu(i,j)=min(flu(i,j),0.0)
                  fhu(i,j)=min(fhu(i,j),0.0)
               else
                  flu(i,j)=max(flu(i,j),0.0)
                  fhu(i,j)=max(flu(i,j),0.0)
               endif
            endif
!
            if(g(i,j) .GT. g(i+1,j)+h(i+1,j))then
               if(w(i+1,j) .GE. 0.0)then
                  flu(i,j)=max(flu(i,j),0.0)
                  fhu(i,j)=max(fhu(i,j),0.0)
               else
                  flu(i,j)=min(flu(i,j),0.0)
                  fhu(i,j)=min(fhu(i,j),0.0)
               endif
            endif
!
            if(h(i,j)+g(i,j) .LT. g(i,j+1))then
               if(w(i,j+1) .GE. 0.0)then
                  flv(i,j)=min(flv(i,j),0.0)
                  fhv(i,j)=min(fhv(i,j),0.0)
               else
                  flv(i,j)=max(flv(i,j),0.0)
                  fhv(i,j)=max(fhv(i,j),0.0)
               endif
            endif
!
            if(g(i,j) .GT. g(i,j+1)+h(i,j+1))then
               if(w(i,j+1) .GE. 0.0)then
                  flv(i,j)=max(flv(i,j),0.0)
                  fhv(i,j)=max(fhv(i,j),0.0)
               else
                  flv(i,j)=min(flv(i,j),0.0)
                  fhv(i,j)=min(fhv(i,j),0.0)
               endif
            endif
! 
        end do
        end do
!
     return
     end subroutine bothfluxschemes_w
