      subroutine entrainment
!***************************************************************************
!*
!*    This routine calculates both the mass and momentum
!*    coupling between the dense layer and the leading edge. Also
!*    the top entrainment. Is uses the arrays fhu
!*    and fhv to hold the force terms. 
!*    winduh and windvh are the u- and v- components of wind
!*    at height hnew(i,j).
!*
!***************************************************************************
      use master_variables
      implicit none
!
      integer(kind=4) :: i,j,ie,je
      real   (kind=8) :: uedgetot,sum,z,winduh,windvh,umod,fact,termu,termv,dhudt,dhvdt,dhdt
      real   (kind=8) :: visx,visy,kappau,kappad,kappal,kappar,gdashstar,vee,stab,zrou
      real   (kind=8) :: uedge,richard,k,betatemp,dtdx,dtdy,dxinv,dyinv
!
!***  Initializations
!
      uedgetot = 0.0
      sum      = 0.0
      dtdx     = dt/dx
      dtdy     = dt/dy 
      dxinv    = 1.0/dx
      dyinv    = 1.0/dy
!
      do j = ny1,ny2
         do i = nx1,nx2 
!
            z    = hnew(i,j)
            zrou = zrough(i,j)
            if(z.gt.zrou)then
!            
                umod = sqrt(windu(i,j)*windu(i,j)+windv(i,j)*windv(i,j))
                fact = log(z/zrou)-stab(z,rmonin(i,j))+stab(zrou,rmonin(i,j))
                fact = max(fact,0.0)
                winduh = ustar(i,j)*fact*windu(i,j)/(umod*vkarm)
                windvh = ustar(i,j)*fact*windv(i,j)/(umod*vkarm)
!
            else
               winduh = 0.0
               windvh = 0.0
            endif
!            
            if(hnew(i+1,j) .GT. hnew(i-1,j))then
               ie=i-1
            else
               ie=i+1
            endif
!
            if(hnew(i,j+1) .GT. hnew(i,j-1))then
               je=j-1
            else
               je=j+1
            endif
!
            termu= winduh*(i-ie)*((hnew(ie,j )*(unew(ie,j )-winduh)) -  &
                                  (hnew(i ,j )*(unew(i ,j )-winduh)))+  &
                   windvh*(j-je)*((hnew(i ,je)*(unew(i ,je)-winduh)) -  &
                                  (hnew(i ,j )*(unew(i ,j )-winduh)))
!
            termv= winduh*(i-ie)*((hnew(ie,j )*(vnew(ie,j )-windvh)) -  &
                                  (hnew(i ,j )*(vnew(i ,j )-windvh)))+  &
                   windvh*(j-je)*((hnew(i ,je)*(vnew(i ,je)-windvh)) -  &
                                  (hnew(i ,j )*(vnew(i ,j )-windvh)))
!
            dhudt = hnew(i,j)*(unew(i,j)-winduh)-h(i,j)*(u(i,j)-winduh)
            dhvdt = hnew(i,j)*(vnew(i,j)-windvh)-h(i,j)*(v(i,j)-windvh)
!
            fhu(i,j) = dhudt-termu*dtdx
            fhv(i,j) = dhvdt-termv*dtdy
!
!***        \diff{}{t}[h(u-u_a)]-[(u_a)\dot \nabla ][h(u-u_a)]
!
            dhdt =  (hnew(i,j)-h(i,j))                              &
                   -(winduh*(i-ie))*(hnew(ie,j )-hnew(i,j))*dtdx    &
                   -(windvh*(j-je))*(hnew(i ,je)-hnew(i,j))*dtdy
!
            if(dhdt .LT. 0.0)then
               fhu(i,j)= 0.0
               fhv(i,j)= 0.0
            endif
!
!***        Dissipation terms
!***        The x- and y- components of the viscous force are here called
!***        visx and visy. Note that, because of the dyadic in the term,
!***        that visx is a function of only `u' and visy is a function only
!***        of `v'. 
!
            if(hnew(i,j) .GT. zrou)then

               kappau = hnew(i  ,j+1)*sqrt(unew(i  ,j+1)*unew(i  ,j+1)+vnew(i  ,j+1)*vnew(i  ,j+1)) 
               kappad = hnew(i  ,j-1)*sqrt(unew(i  ,j-1)*unew(i  ,j-1)+vnew(i  ,j-1)*vnew(i  ,j-1)) 
               kappal = hnew(i-1,j  )*sqrt(unew(i-1,j  )*unew(i-1,j  )+vnew(i-1,j  )*vnew(i-1,j  )) 
               kappar = hnew(i+1,j  )*sqrt(unew(i+1,j  )*unew(i+1,j  )+vnew(i+1,j  )*vnew(i+1,j  )) 
!
               visx = 4.0*zeta*                                  & 
                    ((kappar*(unew(i+2,j  )-unew(i  ,j  )))-    &
                     (kappal*(unew(i  ,j  )-unew(i-2,j  )))+    &
                     (kappau*(unew(i  ,j+2)-unew(i  ,j  )))-    &
                     (kappad*(unew(i  ,j  )-unew(i  ,j-2))) )
!
               visy = 4.0*zeta*                                  &
                    ((kappar*(vnew(i+2,j  )-vnew(i  ,j  )))-    &
                     (kappal*(vnew(i  ,j  )-vnew(i-2,j  )))+    &
                     (kappau*(vnew(i  ,j+2)-vnew(i  ,j  )))-    &
                     (kappad*(vnew(i  ,j  )-vnew(i  ,j-2))))
!
               flu(i,j) = visx*dt
               flv(i,j) = visy*dt
!
            endif
!
!***        Top entrainment terms
!***        uedge is the local edge entrainment speed
!***        and uedgetot is the total amount of entrainment summed over the whole cloud.
!
            if(hnew(i,j) .GT. zrou .AND. rhonew(i,j) .GT. rhoa) then
!
               gdashstar = gi*max((rhonew(i,j)-rhoa)/rhoa,1.0)
               vee = sqrt(xal2*wstar(i,j)*wstar(i,j)+ustar(i,j)*ustar(i,j)+                  &
                          cd(i,j)*0.5*(xal3*xal3*(unew(i,j)*unew(i,j)+vnew(i,j)*vnew(i,j)))+   &
                          xal7*xal7*((unew(i,j)-winduh)*(unew(i,j)-winduh)+                  &
                         (vnew(i,j)-windvh)*(vnew(i,j)-windvh)))
!
               if(vee .GT. zero)then
                  richard  = gdashstar*hnew(i,j)/vee/vee
                  utop(i,j)= vkarm/(1.0+brit*richard)*vee
               else
                  utop(i,j)=0.0
               endif
            else
               unew(i,j) = 0.0
               vnew(i,j) = 0.0
               utop(i,j) = 0.0
            endif
!
            uedge=((hnew(i,j)-h(i,j))+                                      &
                   (winduh*(i-ie)*dxinv)*(hnew(ie,j)-hnew(i,j))*dt+            &
                   (windvh*(j-je)*dyinv)*(hnew(i,je)-hnew(i,j))*dt )*train

            uedge = max(uedge,0.0)/dt
            if(uedge .LT. 0.0) call runend(-1,'Negative value uedge')
            
            uedgetot = uedgetot+uedge*dt
            sum      = sum+hnew(i,j)
!
         end do
         end do
!
      if(sum .GT. zero)then
          do j = ny1,ny2
             do i = nx1,nx2
               if(hnew(i,j) .GT. zrough(i,j))then
                  rhonew(i,j)=(rhonew(i,j)*hnew(i,j)+rhoa*hnew(i,j)*uedgetot/sum)/ &
                              (hnew(i,j)+hnew(i,j)*uedgetot/sum)
                  hnew(i,j)=hnew(i,j)*(1.0+uedgetot/sum)
               endif
             end do
         end do
      endif
!
      betatemp=beta
      k=1.0/(2.0*shape*betatemp*betatemp)
!
!***  Final contribution from all terms
!
      do j = ny1,ny2
         do i = nx1,nx2
!
            if(hnew(i,j) .GT. zrough(i,j))then
              unew(i,j)=unew(i,j)-k*rhoa*fhu(i,j)/(rhonew(i,j)*hnew(i,j))
              vnew(i,j)=vnew(i,j)-k*rhoa*fhv(i,j)/(rhonew(i,j)*hnew(i,j))
!
              unew(i,j)=unew(i,j)-flu(i,j)
              vnew(i,j)=vnew(i,j)-flv(i,j)
!
              unew(i,j)=(unew(i,j)*hnew(i,j)*rhonew(i,j) + winduh*dt*utop(i,j)*rhoa)/  &
                        (hnew(i,j)*rhonew(i,j)+dt*utop(i,j)*rhoa)
              vnew(i,j)=(vnew(i,j)*hnew(i,j)*rhonew(i,j) + windvh*dt*utop(i,j)*rhoa)/  &
                        (hnew(i,j)*rhonew(i,j)+dt*utop(i,j)*rhoa)
!
              rhonew(i,j)=(rhonew(i,j)*hnew(i,j)+rhoa*dt*utop(i,j))/(hnew(i,j)+dt*utop(i,j))
              hnew  (i,j)=hnew(i,j)+utop(i,j)*dt
            endif
         end do
       end do
!
      return
      end subroutine entrainment
