       subroutine get_par_ABL(z0,z1,Tz0,Tz1,pres,u,v, &
                              ustar,wstar,rmonin)   
!*****************************************************************************
!*
!*     This routine computs parameters of the ABL
!* 
!*     INPUTS:
!*      z0       Reference level 1
!*      z1       Reference level 2 (z1 > z0)
!*      z0       Temperature (K) at z0
!*      z1       Temperature (K) at z1
!*      pres     Pressure (Pa) 
!*      u        u-velocity (at z1)
!*      v        v-velocity (at z1)
!*     OUTPUTS:
!*      ustar    
!*      wstar    = 0 in this version
!*      rmonin   Monin-Obukhov lenght
!*
!*****************************************************************************
       implicit none
!
       real(kind=8) :: z0,z1,Tz0,Tz1,pres,u,v
       real(kind=8) :: ustar,wstar,rmonin
!
       real(kind=8),parameter :: g = 9.81
       real(kind=8),parameter :: k = 0.4
       real(kind=8) :: Thz0,Thz1,umod,Ri,Gm,Gh,thstar
!
!***   Potential temperature Theta=T*(1bar/P))**(R/cp)
!***   R  =  287  J/kgK   Air specific gas  constant
!***   cp = 1006  J/kgK   Air specific heat capacity
!
       Thz0 = Tz0*(1d5/pres)**(0.285d0)   
       Thz1 = Tz1*(1d5/pres)**(0.285d0)   
!
!***   Velocity
!
       umod = max(sqrt(u*u+v*v),1e-6)
!
!***   Bulk Richardson number
!
       Ri = g*(z1-z0)*(Thz1-Thz0)/(umod*umod*0.5d0*(Tz0+Tz1))
!
!***   Stable/Unstable ABL   
!
       if(Ri.ge.0.0) then
!                             Stable
          Gm = 1.0+4.7d0*Ri
          Gm = 1.0/(Gm*Gm)
          Gh = Gm
       else
!                             Unstable
          Gm = log(z1/z0)
          Gm = 1.0 + (70d0*k*k*sqrt(abs(Ri)*z1/z0))/(Gm*Gm)
          Gh = 1.0 + (50d0*k*k*sqrt(abs(Ri)*z1/z0))/(Gm*Gm)
          Gm = 1.0 - ((9.4d0*Ri)/Gm)
          Gh = 1.0 - ((9.4d0*Ri)/Gh)
       end if
!
!***   ustar
!	   
       ustar = log(z1/z0)
       ustar = k*umod*sqrt(Gm)/ustar
!
!***   thstar (Pr=1 assumed)
!
       thstar = log(z1/z0)
       thstar = k*k*umod*(Thz1-Thz0)*Gh/(ustar*thstar*thstar)
!
!***   wstar
!
       wstar = 0.0
!
!***   Monin-Obukhov
!
       rmonin = ustar*ustar*0.5d0*(Thz0+Thz1)/(k*g*thstar)
!
       return
     end subroutine get_par_ABL
