      subroutine readdwm
!*************************************************************************************
!*
!*    This routine reads meteorological data from a DWM file at current time
!*
!*    OUTPUTS
!*       meteo_time
!*       windu,windv,ustar,rmonin,wstar(=0),cd
!*
!*    DWM FILE FORMAT
!*         time1(s) time2(s) Tz0(C) Tzref(C) pres(HPa)
!*         windu(m/s)  (1:nx,1:ny) 
!*         windv(m/s)  (1:nx,1:ny)
!*
!*************************************************************************************
        use master_variables
        use io_parameters
        implicit none
!
        logical            :: go_on 
        integer  (kind=4)  :: i,j,nxx,nyy
        real     (kind=4)  :: r4_time1,r4_time2,r4_Tz0,r4_Tzref,r4_pres
        real     (kind=4)  :: r4_dx,r4_dy,r4_x0,r4_y0
        real     (kind=8)  :: time1,time2 
        real     (kind=8)  :: get_cd
!
!***  Opens the DWM binary file
!
        open(90,FILE=TRIM(fdia),STATUS='unknown',ERR=100,FORM='unformatted')
        read(90,err=101) nxx,nyy,r4_dx,r4_dy,r4_x0,r4_y0
!
        if(nxx.ne.nx)         call runend(-1,'Incongruous nx in dwm file')
        if(nyy.ne.ny)         call runend(-1,'Incongruous ny in dwm file')
        if(ABS(DBLE(r4_dx)-dx).gt.0.1) call runend(-1,'Incongruous dx in dwm file')
        if(ABS(DBLE(r4_dy)-dy).gt.0.1) call runend(-1,'Incongruous dy in dwm file')
        if(ABS(DBLE(r4_x0)-x0).gt.0.1) call runend(-1,'Incongruous x0 in dwm file')
        if(ABS(DBLE(r4_y0)-y0).gt.0.1) call runend(-1,'Incongruous y0 in dwm file')
!
!***  Reads the file (using real*4 variables)
!
        go_on = .true.
        do while(go_on)
           read(90,err=101) r4_time1,r4_time2,r4_Tz0,r4_Tzref,r4_pres
           read(90,err=101) work4(1:nx,1:ny,1)
           read(90,err=101) work4(1:nx,1:ny,2)
!
           time1 = DBLE(r4_time1)  
           time2 = DBLE(r4_time2)  
           if((time.ge.time1).and.(time.le.time2)) then
              go_on      = .false.
              meteo_time = time2
!
              Tz0   = DBLE(r4_Tz0  )
              Tzref = DBLE(r4_Tzref)
              pres  = DBLE(r4_pres )
!
              windu(1:nx,1:ny) = DBLE(work4(1:nx,1:ny,1))
              windv(1:nx,1:ny) = DBLE(work4(1:nx,1:ny,2))
!
              pres   = pres*1d2                  ! HPa --> Pa
              Tz0    = Tz0   + 273.              ! C  --> K
              Tzref  = Tzref + 273.              ! C  --> K	 
           end if
!
        end do
!
!***  Computes the rest of meteorological variables at the ABL
!***  ustar,wstar,rmonin
!    
        do j=1,ny
           do i=1,nx
              call get_par_ABL(zrough(i,j),zref,Tz0,Tzref,pres,windu(i,j),windv(i,j), &
                   ustar(i,j),wstar(i,j),rmonin(i,j))            
! 
              cd(i,j) = get_cd(ustar(i,j),windu(i,j),windv(i,j))
!
           end do
        end do
!
        close(90)
        return
!
!***  List of errors
!
100     call runend(-1,'Error opening file'//TRIM(fdia))
101     call runend(-1,'Error reading file'//TRIM(fdia))
        return
      end subroutine readdwm

