      subroutine readuni
!*************************************************************************************
!*
!*    This routine reads meteorological data from a meteo
!*    file at current time
!*
!*    OUTPUTS
!*       meteo_time
!*       windu,windv,ustar,rmonin,wstar(=0),cd
!*
!*    METEO FILE FORMATS
!*       HEADER      : iyr,imo,idy,ihr,imi,code
!*       
!*       CUP   format: time1(s) time2(s) windu(m/s) windv(m/s) Tz0(C) Tzref(C) pres(HPa)
!*       SONIC format: time1(s) time2(s) windu(m/s) windv(m/s) Tzref(C) ustar rmonin
!*
!*************************************************************************************
      use master_variables
      use io_parameters
      implicit none
!
      logical            :: go_on 
      character(len=100) :: message
      integer  (kind=4)  :: istat
      integer  (kind=4 ) :: iyr,imo,idy,ihr,imi 
      real     (kind=8 ) :: time1,time2 
      real     (kind=8 ) :: windu_uni,windv_uni,ustar_uni,wstar_uni,rmonin_uni,cd_uni
      real     (kind=8 ) :: xst,yst,zrou
      real     (kind=8 ) :: get_cd,get_val
      character(len =10) :: code
!
!***  Reads the file header in the form YYYY MM DD HH MM code
!
      open(99,FILE=TRIM(fmet),STATUS='unknown',ERR=100)
      read(99,*,err=101) iyr,imo,idy,ihr,imi,code
!
      if(iyr.ne.ibyr) call runend(-1,'Incongrous begin times in meteo and input files')
      if(imo.ne.ibmo) call runend(-1,'Incongrous begin times in meteo and input files')
      if(idy.ne.ibdy) call runend(-1,'Incongrous begin times in meteo and input files')
      if(ihr.ne.ibhr) call runend(-1,'Incongrous begin times in meteo and input files')
      if(imi.ne.ibmi) call runend(-1,'Incongrous begin times in meteo and input files')
!
      if(TRIM(code).eq.'CUP'.or.TRIM(code).eq.'cup') then
!
!***  CUP anemometer
!
        go_on = .true.
        do while(go_on)
           read(99,*,err=101) time1,time2,windu_uni,windv_uni,Tz0,Tzref,pres
                   if((time.ge.time1).and.(time.le.time2)) then
                   go_on      = .false.
              meteo_time = time2
!
              pres   = pres*1d2                      ! HPa --> Pa
              Tz0    = Tz0   + 273.                  ! C  --> K
              Tzref  = Tzref + 273.                  ! C  --> K	 
              end if
         end do
!
!***    Computes the rest of meteorological variables at the ABL
!***    ustar,wstar,rmonin
!***    First evaluates zroughness at the station coordinates    
!
        call get_input_rea8(finp,'METEO','X_STATION_(UTM_M)',xst,1,istat,message)
        if(istat.ne.0) call runend(-1,message)
!
        call get_input_rea8(finp,'METEO','Y_STATION_(UTM_M)',yst,1,istat,message)
        if(istat.ne.0) call runend(-1,message)
!
        zrou = get_val(x0,y0,nx,ny,dx,dy,zrough,xst,yst)
!
       call get_par_ABL(zrou,zref,Tz0,Tzref,pres,windu_uni,windv_uni,& 
                        ustar_uni,wstar_uni,rmonin_uni)            
!
      else if(TRIM(code).eq.'SONIC'.or.TRIM(code).eq.'sonic') then
!
!***  SONIC anemometer
!
        go_on = .true.
        wstar_uni = 0d0
           do while(go_on)
           read(99,*,err=101) time1,time2,windu_uni,windv_uni,Tzref,ustar_uni,rmonin_uni
            if((time.ge.time1).and.(time.le.time2)) then
                go_on      = .false.
              meteo_time = time2
!
              Tzref  = Tzref + 273.              ! C  --> K
              end if
            end do
!
      else
       call runend(-1,'Invalid anemometer code in meteo file')
      end if 
!
!***  Skin friction (drag coefficient). Set to constant
!
      cd_uni = get_cd(ustar_uni,windu_uni,windv_uni)
!
!***  Assigns uniform values to the arrays
!
      windu (1:nx,1:ny) = windu_uni
      windv (1:nx,1:ny) = windv_uni
      ustar (1:nx,1:ny) = ustar_uni
      wstar (1:nx,1:ny) = wstar_uni
      rmonin(1:nx,1:ny) = rmonin_uni
      cd    (1:nx,1:ny) = cd_uni
!
      close(99)
      return
!
!***  List of errors
!
 100  call runend(-1,'Error opening file'//TRIM(fmet))
 101  call runend(-1,'Error reading file'//TRIM(fmet))
      return
      end subroutine readuni
