      subroutine setup
!**************************************************************************
!*
!*    Sets up the run
!*
!**************************************************************************
      use io_parameters
      use master_variables
      implicit none
!
      integer(kind=4)   :: iarg
      character(len=10) :: stime
      character(len=8 ) :: sdate
!
!***  Gets input and log filenames from the call arguments
!
      iarg = 1                          
      call GETARG(iarg,finp)
      iarg = 2                          
      call GETARG(iarg,flog)
!
!***  Opens the log file
!
      open (nlog,file=TRIM(flog),status='unknown')
!
      call DATE_AND_TIME(sdate,stime)
      write(nlog,1) sdate(7:8),sdate(5:6),sdate(1:4),  &
           stime(1:2),stime(3:4),stime(5:6)
1     format(/,                         &
           '----------------',/, &
           ' TWODEE PROGRAM ',/, &
           '----------------',/, &
           '                ',/, &
           '  Starting date ',a2,'-',a2,'-',a4,' at time ',a2,':',a2,':',a2)
!
!***  Initializes variables
!
      call inivar
!
!***  Reads TIME BLOCK form input file
!
      call readtim 
!
!***  Reads GRID BLOCK from input file 
!
      call readgrd
!
!***  Reads PROPERTIES BLOCK from input file
!
      call readpro
!
!***  Reads METEO BLOCK from input file
!
      call readmet
!
!***  Reads OUTPUT BLOCK from input file
!
      call readout
!
!***  Reads FILES BLOCKS from input file
!
      call readfil
!
!***  Reads NUMERIC BLOCK from input file
!
      call readnum
!
!***  Allocates memory
!
      call getmem
!
!***  Creates (or reads from a regional file) the topography 
!
      call readtop 
!
!***  Reads zrougness from a regional file 
!
      call readem(frou,x0,y0,dx,dy,nx,ny,zrough)
!
!***  If necessary, reads the tracking points file
!
      if(out_pts) then
         call setpts
      else
         npts = 0
      end if
!
!***  If necessary, reads the tracking boxes file
!
      if(out_box) then
        call setbox
      else
        nbox = 0
      end if
!
!***  Writes input data
!
      call wridat
!
!***  Set the initial conditions on variables (h,u,v,rho) as constant or 
!***  as given from a restart file
!
      call setvar
! 
!***  Reads the source file (measured matrix) 
!
      call setsrc 
!
      return 
    end subroutine setup
    
