     subroutine setvar
!****************************************************************
!*
!*   Set the initial conditions on variables (h,u,v,rho) as 
!*   constant or as given from a restart file
!*
!****************************************************************
     use io_parameters
     use master_variables
     implicit none
!
     character(len=1):: cdummy
     integer(kind=4) :: ix,iy,nxx,nyy
     real   (kind=8) :: x00,y00,dxx,dyy
!
     if(restart) then
!
!*** Restart from a file
!
        open(90,FILE=TRIM(frst),STATUS='unknown',ERR=100)
        do ix = 1,7
           read(90,*,err=101) cdummy
        end do
        read(90,*,err=101) time,nxx,nyy,dxx,dyy,x00,y00
!
        if(nxx.ne.nx) call runend(-1,'Incongruous nx in restart file'//TRIM(frst))
        if(nyy.ne.ny) call runend(-1,'Incongruous ny in restart file'//TRIM(frst))
        if(x00.ne.x0) call runend(-1,'Incongruous x0 in restart file'//TRIM(frst))
        if(y00.ne.y0) call runend(-1,'Incongruous x0 in restart file'//TRIM(frst))
        if(dxx.ne.dx) call runend(-1,'Incongruous dx in restart file'//TRIM(frst))
        if(dyy.ne.dy) call runend(-1,'Incongruous dy in restart file'//TRIM(frst))
! 
        do iy=1,ny
        read(90,*,err=101) (h(ix,iy),ix=1,nx)
        end do
        do iy=1,ny
          read(90,*,err=101) (u(ix,iy),ix=1,nx)
        end do
        do iy=1,ny
           read(90,*,err=101) (v(ix,iy),ix=1,nx)
        end do
        do iy=1,ny
        read(90,*,err=101) (rho(ix,iy),ix=1,nx)
        end do
        close(90)   
!
        ocounter = INT(time/tevery)+1
        else
!
!*** Do not restart
!
       h  (1:nx,1:ny) = 0.0
       u  (1:nx,1:ny) = 0.0
       v  (1:nx,1:ny) = 0.0 
       rho(1:nx,1:ny) = rhoa
!
       end if
!
!*** Other initializations
!
     rhonew(1:nx,1:ny) = rho(1:nx,1:ny)
!
!*** Prints to the log file
!
     write(nlog,10) MAXVAL(h  (1:nx,1:ny)),MINVAL(h  (1:nx,1:ny)),SUM(h  (1:nx,1:ny))/(nx*ny), &
                 MAXVAL(u  (1:nx,1:ny)),MINVAL(u  (1:nx,1:ny)),SUM(u  (1:nx,1:ny))/(nx*ny), &
                 MAXVAL(v  (1:nx,1:ny)),MINVAL(v  (1:nx,1:ny)),SUM(v  (1:nx,1:ny))/(nx*ny), &
                 MAXVAL(rho(1:nx,1:ny)),MINVAL(rho(1:nx,1:ny)),SUM(rho(1:nx,1:ny))/(nx*ny)
 10   format(/,  &
        'INITIAL VALUES'     ,/,   &
        '  h   Minimum  :',e12.4,/,  &
        '  h   Maximum  :',e12.4,/,  &
        '  h   Average  :',e12.4,/,  &
        '                '      ,/,  &
        '  u   Minimum  :',e12.4,/,  &
        '  u   Maximum  :',e12.4,/,  &
        '  u   Average  :',e12.4,/,  &
        '                '      ,/,  &
        '  v   Minimum  :',e12.4,/,  &
        '  v   Maximum  :',e12.4,/,  &
        '  v   Average  :',e12.4,/,  & 
        '                '      ,/,  &
        '  rho Minimum  :',e12.4,/,  &
        '  rho Maximum  :',e12.4,/,  &
        '  rho Average  :',e12.4)
!
     return
!
!*** List of errors
!
 100 call runend(-1,'Error opening file'//TRIM(frst))
 101 call runend(-1,'Error reading file'//TRIM(frst))
!      
     return
     end subroutine setvar

