      subroutine writim
!***************************************************************************
!*
!*    Prints time evolution to the log file
!*
!**************************************************************************
        use master_variables
        use io_parameters
        implicit none
!
        integer(kind=4) :: iyr,imo,idy,ihr,imi,ise,i,j
        real   (kind=8) :: summ,xmean,ymean,xarea
        real   (kind=8) :: hmean,summc,c_z
!
!***  Initializations
!         
        summ  = 0.0
        xmean = 0.0
        ymean = 0.0
        xarea = 0.0
        hmean = 0.0
        summc = 0.0
!
        do j=1,ny
           do i=1,nx
              summ  = summ   + h(i,j)
              xmean = xmean  + h(i,j)*i
              ymean = ymean  + h(i,j)*j
              if(h(i,j) .GT. zrough(i,j)) xarea = xarea + 1.0
!
              c_z = c_bg + (1d6-c_bg)*(rho(i,j)-rhoa)/(rho1-rhoa)   ! concentration in ppm
              summc = summc + c_z
              hmean = hmean + c_z*h(i,j) 
!
           end do
        end do
!
        if( summ .GT. 1d-3)then
           xmean = xmean*dx/summ
           ymean = ymean*dy/summ
        end if 
        xarea = xarea*dx*dy
!
!***  Writes to the log file
!
      call addtime(ibyr,ibmo,ibdy,ibhr,ibmi,iyr,imo,idy,ihr,imi,ise,time)
!
      write(nlog,10) icounter,dt,int((time-tbeg)/60.0),        &
                     ((time-tbeg)/(trun-tbeg))*1d2,           &
                     idy ,month(imo ),iyr ,ihr ,imi ,ise
   10 format(/,                                                       &
          '-> Iteration                 : ',i8                   ,/,  &
          '   Time step                 : ',e13.6                ,/,  &
          '   Ellapsed time             : ',i7,' min   (',f7.2,'%)',/,  &
          '   Current time              : ',i2,1x,a3,1x,i4,' at ',i2,':',i2,':',i2 ,/, &
          '   -------                     ')
!
      write(nlog,20)  MAXVAL(h),MINVAL(h),hmean/summc, &
                      xmean,ymean,xarea,1d2*xarea/(nx*ny*dx*dy)  
   20 format( &
          '   Max h     (m )            : ',f12.4,/,  &
          '   Min h     (m )            : ',f12.4,/,  &
          '   Average h (m )            : ',f12.4,/,  &
          '   X-mean    (m )            : ',f12.1,/,  &
          '   Y-mean    (m )            : ',f12.1,/,  &
          '   Area      (m2)            : ',f12.1,' (',f7.2,'%)',/,&
          '   -------                     ')
!
!***
!
        call flush(nlog)
!
!***  Writes tracking points
! 
        if(out_pts) then
           call wripts
        end if
!
!***  Writes tracking boxes
! 
        if(out_box) then
           call wribox
        end if
!
        return
      end subroutine writim
